#!/usr/bin/env python
import visvis as vv
from visvis import Aarray
app = vv.use()

# Let's say we have lena, but only the even pixels in the y dimension.
# So each pixel should have twice the size in the y direction.
im = vv.imread('astronaut.png')
im = im[::2,:,:]

# Init a figure with two axes
vv.figure()
a1 = vv.subplot(121); vv.title('pixel units')
a2 = vv.subplot(122); vv.title('real-world units')

# Method 1: scale the whole scene
# Use this if you want the axis to depict pixel units.
t1 = vv.imshow(im, axes=a1)
a1.daspect = 1,-2 # daspect works x,y,z, the y-axis is flipped for images

# Method 2: use the Aarray class to scale the image
# You could use this is you know the physical dimensions of a pixel,
# to have the axis depict, for example, mm.
im2 = Aarray(im,(2,1,1))  # sampling is given in y,x,color order
t2 = vv.imshow(im2, axes=a2)

app.Run()



import os
import sys
import pytest

@pytest.fixture(scope="session", autouse=True)
def execute_before_any_test():
    if os.getenv('TEST_INSTALL') not in ('1', 'true'):
        sys.path.insert(0, '..')



import ssdf

# Load from file
s1 = ssdf.load('test1.ssdf')

# Write to string and read back
tmp = ssdf.saves(s1)
s2 = ssdf.loads(tmp)

# Print, manually inspection required to see whether it matches the test file
print(s2)



import os


def test_im_read_write():
    import visvis as vv
    im = vv.imread('astronaut.png')
    assert im.shape == (512, 512, 3)
    vv.imwrite(os.path.expanduser('~/astronaut2.png'), im)


def test_mesh_read_write():
    import visvis as vv
    m = vv.meshRead('bunny.ssdf')
    assert isinstance(m, vv.BaseMesh)
    vv.meshWrite(os.path.expanduser('~/bunny2.stl'), m)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# SSDF is distributed under the terms of the (new) BSD License.
# See http://www.opensource.org/licenses/bsd-license.php

# This module was first called points.py. In this newer version, PEP8 is
# followed more strictly, in order to make it easier to adopt by others.

""" Module pypoints

Provides several classes to represent points, pointsets, anisotropic arrays,
and quaternions.


Usage
-----
from pypoints import Point, Pointset, Aarray, Quaternion


The Point and Pointset classes
------------------------------
A point represents a location or vector. The dimension of the point
can be of any dimension.

A pointset represents an ordered (list) of points.
It has methods to add and remove points. Both point instances
and pointset instances have methods for mathematical operations, like
for example calculating distances between points and cross correlation.
For the Pointset these operations are efficiently applied to all points.


The Aarray class
----------------
The Aarray class implements numpy.ndarray and provides a way to manage
anisotropic data.


The Quaternion class
--------------------
A Quaternion is a method to describe and work with rotations. It avoids
the problem of Gimbal lock.


Module compatibility
--------------------
Use the is_* methods to test for points, pointsets etc, this will
make different packages using this module compatible.

from pypoints import is_Point, is_Pointset, is_Aarray, is_Quaternion


More information and license
----------------------------
For more information see the docstrings of the classes.

Copyright (C) 2012, Almar Klein.
This software distributed under the BSD license.

"""

# Set version number
__version__ = 2.2

import numpy as np
import sys


# todo: mention in next release notes that warning is not displayed by default
SHOW_SUBTRACTBUG_WARNING = False


def getExceptionInstance():
    type, value, tb = sys.exc_info()
    del tb
    return value


def Property(function):
    """ Property(function)
    
    A property decorator which allows to define fget, fset and fdel
    inside the function.
    
    Note that the class to which this is applied must inherit from object!
    Code based on an example posted by Walker Hale:
    http://code.activestate.com/recipes/410698/#c6
    
    """
    
    # Define known keys
    known_keys = 'fget', 'fset', 'fdel', 'doc'
    
    # Get elements for defining the property. This should return a dict
    func_locals = function()
    if not isinstance(func_locals, dict):
        raise RuntimeError('Property function should "return locals()".')
    
    # Create dict with kwargs for property(). Init doc with docstring.
    D = {'doc': function.__doc__}
    
    # Copy known keys. Check if there are invalid keys
    for key in func_locals.keys():
        if key in known_keys:
            D[key] = func_locals[key]
        else:
            raise RuntimeError('Invalid Property element: %s' % key)
    
    # Done
    return property(**D)


def nearest_power_of_two(i):
    """ nearest_power_of_two(i)
    Returns the nearest power of two that is at least i.
    """
    ii = 1
    while ii<i:
        ii*=2
    return ii


def check_the_two(p1,p2, what='something'):
    """ check_the_two(p1,p2, what='something')
    Check if the two things (self and a second point/pointset)
    can be used to calculate stuff.
    Returns (p1,p2), if one is a point, p1 is it.
    """
    
    # if one is a point, put it in p1
    if is_Pointset(p1):
        p2,p1 = p1,p2
    
    # only pointsets of equal length can be used
    if not is_Point(p1) and len(p1) != len(p2):
        # define errors
        tmp = "To calculate %s between two pointsets, " % what
        err = tmp + "both pointsets must be of equal length."
        raise ValueError(err)
    
    # check dimensions
    if p1.ndim != p2.ndim:
        tmp = "To calculate %s between two pointsets, " % what
        err = tmp + "the dimensions must be equal."
        raise ValueError(err)
    
    return p1, p2


# Methods to replace isinstance, so this module becomes compatible with
# other version
def is_Point(ob):
    return hasattr(ob, '_is_Point')
def is_Pointset(ob):
    return hasattr(ob, '_is_Pointset')
def is_Point_or_Pointset(ob):
    return hasattr(ob, '_is_Point') or hasattr(ob, '_is_Pointset')
def is_Aarray(ob):
    return hasattr(ob, '_is_Aarray')
def is_Quaternion(ob):
    return hasattr(ob, '_is_Quaternion')


class BasePoints(object):
    """ BasePoints
    
    Abstract class for the Point and Pointset classes. It defines
    addition, substraction, multiplication and devision for points and
    pointsets, and it implements some mathematical methods.
    
    """
    
    ## Common stuff (of which most is overloaded)
    
    @property
    def ndim(self):
        """ Get the number of dimensions.
        """
        return self._data.shape[-1]
    
    @property
    def data(self):
        """ Get the internal numpy array.
        """
        raise NotImplementedError()
    
    def copy(self):
        """ copy()
        
        Make a copy of the point or pointset.
        
        """
        raise NotImplementedError()
    
    def __setitem__(self):
        raise NotImplementedError()
    
    def __getitem__(self):
        raise NotImplementedError()
    
    
    ## Math stuff on a single vector


    def norm(self):
        """ norm()
        
        Calculate the norm (length) of the vector.
        This is the same as the distance to point 0,0 or 0,0,0,
        but implemented a bit faster.
        
        """
        
        # we could do something like:
        #   return self.distance(Point(0,0,0))
        # but we don't have to perform checks now, which is faster...
        
        data = self.data
        dists = np.zeros( (data.shape[0],) )
        for i in range(self.ndim):
            dists += data[:,i].astype(np.float64)**2
        return np.sqrt(dists)


    def normalize(self):
        """ normalize()
        
        Return normalized vector (to unit length).
        
        """
        
        # calculate factor array
        f = self.norm()
        where = f > 0
        f[where] = 1.0 / f[where]
        f.shape = f.size,1
        f = f.repeat(self.ndim,1)
        
        # apply
        data = self.data * f
        
        # return as point or pointset
        if is_Point(self):
            return Point(data)
        else:
            return Pointset(data)


    def normal(self):
        """ normal()
        
        Calculate the normalized normal of a vector.
        Use (p1-p2).normal() to calculate the normal of the line p1-p2.
        Only works on 2D points. For 3D points use cross().
        
        """
        
        # check dims
        if not self.ndim == 2:
            raise ValueError("Normal can only be calculated for 2D points.")
        
        # prepare
        a = self.copy()
        f = 1.0/self.norm()
        
        # swap xy, y goes minus
        tmp = a.data[:,0].copy()
        a.data[:,0] = a.data[:,1] * f
        a.data[:,1] = -tmp * f
        return a


    ## Math stuff on two vectors
    
    
    def distance(self, *p):
        """ distance(p)
        
        Calculate the Euclidian distance between two points or pointsets.
        Use norm() to calculate the length of a vector.
        
        """
        
        # the point directly given?
        if len(p)==1:
            p = p[0]
        
        # make sure p is a point or poinset
        if not is_Point_or_Pointset(p):
            p = Point(p)
        
        # check
        p1,p2 = check_the_two(self,p,'distance')
        
        # calculate
        dists = np.zeros( (p2.data.shape[0],) )
        for i in range(p1.ndim):
            tmp = p1.data[:,i] - p2.data[:,i]
            dists += tmp.astype(np.float64)**2
        return np.sqrt(dists)


    def angle(self, *p):
        """ angle(p)
        
        Calculate the angle (in radians) between two vectors.
        For 2D uses the arctan2 method so the angle has a sign.
        For 3D the angle is the smallest angles between the two
        vectors.
        
        If no point is given, the angle is calculated relative to the
        positive x-axis.
        
        """
        
        # the point directly given?
        if len(p)==1:
            p = p[0]
        elif len(p)==0:
            # use default point
            p = Point([0 for i in range(self.ndim)])
            p.x = 1
            
        # make sure p is a point or poinset
        if not is_Point_or_Pointset(p):
            p = Point(p)
        
        # check. Keep the correct order!
        check_the_two(self,p,'angle')
        p1,p2 = self, p
        
        if p1.ndim ==2:
            # calculate 2D case
            data1, data2 = p1.data, p2.data
            angs1 = np.arctan2( data1[:,1], data1[:,0] )
            angs2 = np.arctan2( data2[:,1], data2[:,0] )
            dangs =  angs1 - angs2
            # make number between -pi and pi
            I = np.where(dangs<-np.pi)
            dangs[I] += 2*np.pi
            I = np.where(dangs>np.pi)
            dangs[I] -= 2*np.pi
            return dangs
            
        elif p1.ndim ==3:
            # calculate 3D case
            p1, p2 = p1.normalize(), p2.normalize()
            data = p1.dot(p2)
            # correct for round off errors (or we will get NANs)
            data[data>1.0] = 1.0
            data[data<-1.0] = -1.0
            #return data
            return np.arccos(data)
        
        else:
            # not possible
            raise ValueError("Can only calculate angle for 2D and 3D vectors.")
    
    
    def angle2(self, *p):
        """ angle2(p)
        
        Calculate the angle (in radians) of the vector between
        two points.
        
        Notes
        -----
        Say we have p1=(3,4) and p2=(2,1).
        
        p1.angle(p2) returns the difference of the angles of the two vectors:
        0.142 = 0.927 - 0.785
        
        p1.angle2(p2) returns the angle of the difference vector (1,3):
        p1.angle2(p2) == (p1-p2).angle()
        
        """
        
        # the point directly given?
        if len(p)==1:
            p = p[0]
        elif len(p)==0:
            raise ValueError("Function angle2() requires another point.")
        else:
            p = Point(*p)
        
        # check. Keep the correct order!
        check_the_two(self,p,'angle')
        p1,p2 = self, p
        
        if p1.ndim in [2,3]:
            # subtract and use angle()
            return (p1-p2).angle()
            #meaning: dangs = np.arctan2( data2[:,1]-data1[:,1], data2[:,0]-data1[:,0] )
        else:
            # not possible
            raise ValueError("Can only calculate angle for 2D and 3D vectors.")
    
    
    def dot(self, *p):
        """ dot(p)
        
        Calculate the dot product of the two points or pointsets.
        The dot producet is the standard inner product of the
        orthonormal Euclidean space.
        
        """
        
        # the point directly given?
        if len(p)==1:
            p = p[0]
            
        # make sure p is a point or poinset
        if not is_Point_or_Pointset(p):
            p = Point(p)
        
        # check
        p1,p2 = check_the_two(self,p,'dot')
        
        # calculate
        data1, data2 = p1.data, p2.data
        data = np.zeros( (data2.shape[0],) )
        for i in range(p1.ndim):
            tmp = data1[:,i] * data2[:,i]
            data += tmp
        return data
    
    
    def cross(self, *p):
        """ cross(p)
        
        Calculate the cross product of two 3D vectors.
        
        Given two vectors, returns the vector that is orthogonal to
        both vectors. The right hand rule is applied; this vector is
        the middle finger, the argument the index finger, the returned
        vector points in the direction of the thumb.
        
        """
        
        # the point directly given?
        if len(p)==1:
            p = p[0]
        
        if not self.ndim == 3:
            raise ValueError("Cross product only works for 3D vectors!")
        
        # make sure p is a point or poinset
        if not is_Point_or_Pointset(p):
            p = Point(p)
        
        # check (note that we use the original order for calculation)
        p1, p2 = check_the_two(self,p,'cross')
        
        # calculate
        a, b = self.data, p.data
        data = np.zeros( p2.data.shape, np.float32 )
        data[:,0] = a[:,1]*b[:,2] - a[:,2]*b[:,1]
        data[:,1] = a[:,2]*b[:,0] - a[:,0]*b[:,2]
        data[:,2] = a[:,0]*b[:,1] - a[:,1]*b[:,0]
        
        # return
        if is_Pointset(p2):
            return Pointset(data)
        else:
            return Point(data)


    def __add__(self, p):
        """ Add vectors. """
        
        # make sure p is a point or poinset
        if not is_Point_or_Pointset(p):
            p = Point(p)
        
        # check
        p1,p2 = check_the_two(self,p,'add')
        
        # apply and return
        data = p1.data + p2.data # this should go well
        if is_Point(self):
            return Point(data)
        else:
            return Pointset(data)
    
    
    def _showTrace(self):
        # Get traceback info
        type, value, tb = sys.exc_info()
        err = ''
        frame = tb.tb_frame.f_back
        count = 0
        try:
            while frame and count < 5:
                count += 1
                if 'iepRemote' in frame.f_code.co_filename:
                    break
                err +=  "line %i of %s.\n" % (
                        frame.f_lineno, frame.f_code.co_filename)
                frame = frame.f_back
        finally:
            del tb, frame
        print(err[:-1])
    
    
    def __sub__(self, p):
        """ Subtract vectors. """
        
        # make sure p is a point or pointset
        if not is_Point_or_Pointset(p):
            p = Point(p)
        
        # check. Keep the correct order!
        p3, p4 = check_the_two(self,p,'subtract')
        p1, p2 = self, p
        
        # Check for the subtract bug ...
        if SHOW_SUBTRACTBUG_WARNING and p1 is not p3:
            # In previous versions the subtract bug would have occured
            # do not allow...
            try:
                raise ValueError()
            except Exception:
                print('+'*79)
                print('Note that previous versions of pypoints contained a ' +
                'bug in the minus operator. To prevent this message, use the ' +
                'subtract method instead of the operator. To get the ' +
                'same behavior as before, replace "A-B" with "B.subtract(A)". ' +
                'Info: https://github.com/almarklein/visvis/issues/30. ' +
                'Trace:')
                self._showTrace()
                print('+'*79)
        
        # apply and return
        data = p1.data - p2.data # this should go well
        if is_Pointset(p1) or is_Pointset(p2):
            return Pointset(data)
        else:
            return Point(data)
    
    
    def subtract(self, p):
        """ subtract(other)
        
        Subtract Pointset/Point instances.
        
        
        Notes
        -----
        This method was introduced because of the minus-bug. To get the
        same behaviour of when the bug was still there, replace
        "A-B" with B.subtract(A).
        
        """
        
        # make sure p is a point or pointset
        if not is_Point_or_Pointset(p):
            p = Point(p)
        
        # check. Keep the correct order!
        check_the_two(self,p,'subtract')
        p1, p2 = self, p
        
        # apply and return
        data = p1.data - p2.data # this should go well
        if is_Pointset(p1) or is_Pointset(p2):
            return Pointset(data)
        else:
            return Point(data)
    

    def __mul__(self,value):
        """ Scale vectors. """
        
        p1, p2 = self, None
        try:
            value = float(value)
            data1, data2 = self.data, value
        except TypeError:
            
            if not is_Point_or_Pointset(value):
                value = Point(value)
            
            # check
            p1,p2 = check_the_two(self,value,'multiply')
            data1, data2 = self.data, value.data
        
        # apply and return
        data = data1 * data2 # this should go well
        if is_Pointset(p1) or is_Pointset(p2):
            return Pointset(data)
        else:
            return Point(data)
    
    
    def __truediv__(self, value):
        """ Scale vectors. """
        
        p1, p2 = self, None
        try:
            value = float(value)
            data1, data2 = self.data, value
        except TypeError:
            
            if not is_Point_or_Pointset(value):
                value = Point(value)
                
            # check (note that the order is important for division!)
            check_the_two(self,value,'divide')
            data1, data2 = self.data, value.data
            
        # apply and return
        data = data1 / data2 # this should go well
        if is_Pointset(p1) or is_Pointset(p2):
            return Pointset(data)
        else:
            return Point(data)
    
    
    def __rmul__(self, value):
        """ Same as normal multiply. """
        
        return self.__mul__(value)
    
    
    def __rtruediv__(self, value):
        """ Inverse scale vectors. """
        
        p1, p2 = self, None
        try:
            value = float(value)
            data1, data2 = self.data, value
        except TypeError:
            
            if not is_Point_or_Pointset(value):
                value = Point(value)
                
            # check (note that the order is important for division!)
            check_the_two(self,value,'divide')
            data1, data2 = self.data, value.data
            
        # apply and return
        data = data2 / data1 # here's the difference
        if is_Pointset(p1) or is_Pointset(p2):
            return Pointset(data)
        else:
            return Point(data)
    
    __div__ = __truediv__ # For Python 2.x
    __rdiv__ = __rtruediv__


class Point(BasePoints):
    """ Point(x,y,[z,[...]])
    
    Represents a point or vector (of any dimension).
    
    This class implements many usefull operators such as addition
    and multiplication, and provides common mathematical operations
    that can be applied to points and pointsets.
    
    Example
    -------
    p1 = Point(3.2,4)   # a 2D point
    p2 = p1.copy()      # make a copy
    p1[0] = 9           # set the first element
    p1.x                # convenience property (.y and .z also available)
    p1.xi               # idem, but rounded to nearest integer
    p1.distance(p2)     # calculate distance
    p1 + p2             # calculate the addition of the two vectors
    p2 = p1*2           # scale vector
    p2 = p1 * p2        # even for each dimension seperately
    p2 = p2.normalize() # make unit length
    
    """
    
    _is_Point = True
    
    def __init__(self, *point):
        BasePoints.__init__(self)
        
        if len(point)==0:
            raise ValueError("Cannot create an 'empty' point.")
        elif len(point)==1:
            point = point[0]
        
        if is_Point(point):
            # a point
            self._data = point._data.astype(np.float32)
        elif isinstance(point, (tuple,list)):
            # a tuple or list
            self._data = np.array(point, dtype=np.float32)
        elif isinstance(point, np.ndarray):
            # a numpy array
            self._data = point.astype(np.float32)
        elif isinstance(point, (float,int)):
            # 1D point
            self._data = np.empty((1,), dtype=np.float32)
            self._data[0] = point
        else:
            # otherwise, what were we given?
            raise ValueError("Cannot create a point with that argument.")
        
        # remove singleton dimensions
        if self._data.size>1:
            self._data = np.squeeze(self._data)
        
        # check integrity
        if len(self._data.shape) != 1:
            # this must be the case to allow indexing!
            raise ValueError("A point should be given as a 1D array.")
        if self._data.shape[0] < 1:
            raise ValueError("A point should consist of at least one value.")


    def copy(self):
        """ copy()
        
        Get a copy of this point.
        
        """
        return Point(self._data)
    
    
    @property
    def data(self):
        """ Get the point as the (2D) numpy array it is stored in.
        """
        # this is overloaded in the Pointset class
        data = self._data[:]
        data.shape = 1, data.shape[0]
        return data
    

    ## string representation
    
    def __str__(self):
        """ Return a nice string representation of the point. """
        s = "<point  "
        for value in self._data.flatten():
            # '% x.yg': x signif, y length '-1.111e+000'. x = y+7
            s += "% 12.5g, " % (value)
            #str(value).ljust(7,' ') # '%5f' % (value)
        s = s[:-2] + " >"
        return s

    __repr__ = __str__
    
    ## Comparison
    
    def __eq__(self, other):
        """ Test whether two poins are the same. """
        if not is_Point(other) or other.ndim != self.ndim:
            return False
        return (other.data == self.data).sum() == self.ndim
    
    def __ne__(self, other):
        """ Test whether two poins are NOT the same. """
        if not is_Point(other) or other.ndim != self.ndim:
            return True
        return (other.data == self.data).sum() != self.ndim
    
    ## indexing etc
        
    def __setitem__(self,index,value):
        self._data[index] = value

    def __getitem__(self, index):
        return self._data[index]
    
    
    @Property
    def x():
        """ Get/set p[0]. """
        def fget(self):
            return self._data[0]
        def fset(self, value):
            self._data[0] = value
        return locals()
    
    @property
    def xi(self):
        """ Get p[0] rounded to the nearest integer, for indexing. """
        return int(self._data[0]+0.5)
    
    @Property
    def y():
        """ Get/set p[1]. """
        def fget(self):
            return self._data[1]
        def fset(self, value):
            self._data[1] = value
        return locals()
       
    @property
    def yi(self):
        """ Get p[1] rounded to the nearest integer, for indexing. """
        return int(self._data[1]+0.5)
    
    @Property
    def z():
        """ Get/set p[2]. """
        def fget(self):
            return self._data[2]
        def fset(self, value):
            self._data[2] = value
        return locals()
    
    @property
    def zi(self):
        """ Get p[2] rounded to the nearest integer, for indexing. """
        return int(self._data[2]+0.5)



class Pointset(BasePoints):
    """ Pointset(ndim)
    
    Represents a set of points or vectors (of any dimension).
    
    Can also be initialized using: Pointset(<ndim times npoints numpy array>)
    
    A pointset provides an efficient way to store points.
    Internally the points are stored in a numpy array that is
    resized by a factor of two when more space is required. This makes adding
    and removing points (from the end) very efficient. Also mathematical
    operations can be applied on all the points in the set efficiently.
    
    Notes on slicing and indexing
    -----------------------------
      * pp[7]: When indexing, the corresponding Point instance is get/set.
      * pp[7:20]: When slicing, a new poinset (subset) is get/set.
      * pp[4:9,3] When using two indices or slices, the indices are applied to
        the internal data. (In this example returning the z-value for
        points 4 till 8.)
    
    Math
    ----
    The same mathematical operations that can be applied to a Point
    instance can also be applied to a Pointset instance. The operation
    is applied to all points in the set. For example pp.distance(3,4)
    returns an array with the distances of all points in pp to (3,4).
    
    Example
    -------
    a  = <...>          # an existing 100x2 array
    pp1 = Pointset(2)   # pointset with two dimensions
    pp2 = Pointset(a)   # dito
    pp1.append(3,4)     # add a point
    pp1.append(p)       # add an existing point p
    pp1.extend(pp1)     # extend pp1 to itself
    pp2[:4] = pp1       # replace first four points of pp2
    pp[1]               # returns the point (3,4) (as a Point instance)
    pp[1,0]             # returns the value 3.0
    pp[:,1]             # get all y values
    pp.contains(3,4)    # will return True
    
    """
    
    _is_Pointset = True
    
    def __init__(self, ndim):
        BasePoints.__init__(self)
        
        # init
        data = ndim
        initialLength = 16
        
        # convert numpy scalars to int
        if isinstance(data, np.ndarray):
            if data.size==1 and len(data.shape) in [0,1]:
                data = ndim = int(data)
        
        # set using a numpy array
        if isinstance(data, np.ndarray):
            # do some checks
            if len(data.shape) != 2:
                raise ValueError("A pointset should be given as a 2D array.")
            if data.shape[1] < 1:
                tmp = "Each point should consist of at least one value."
                raise ValueError(tmp)
            # create array
            self._len = data.shape[0]
            L = max( nearest_power_of_two(data.shape[0]), initialLength)
            self._data = np.zeros((L, data.shape[1]), dtype=np.float32)
            # fill values
            self._data[:data.shape[0],:] = data
        
        # we were given the dimension
        else:
            self._len = 0
            self._data = np.zeros((initialLength, ndim), dtype=np.float32)
    
    
    @property
    def data(self):
        """ Get a view of the data. Note that internally the points
        are stored in a numpy array that is in general longer than
        the amount of points (at most 4 times as long). This is to
        make adding/removing points much faster by prevening reallocating
        memory. The internal array is resized with a factor two when
        necesary. This property returns a (sub)view of that array and is
        always 2D.
        """
        return self._data[:self._len,:]

    
    def _as_point(self, *p):
        """ _as_point(*p)
        
        Return the input as a point instance, also
        check whether the dimensions match.
        
        """
        
        # the point directly given?
        if len(p)==1:
            p = p[0]
        
        # make sure we are dealing with a point
        if not is_Point(p):
            p = Point(p)
        
        # check whether we can append it
        if self.ndim != p.ndim:
            tmp = "Given point does not match dimension of pointset."
            raise ValueError(tmp)
        
        # done
        return p
    
    
    def _resize_if_required(self, n=None):
        """ _resize_if_required(n=None)
        
        Resize the internal array, n is the new required
        amount of poinst that we should be able to fit in.
        If not given, self._len is used.
        This method checks if we should reduce our size (if
        the internal length is larger than four times n).
        
        """
        
        # n given?
        if n is None:
            n = self._len
        
        # reduce or increase size?
        internalLen = self._data.shape[0]
        if n > internalLen:
            L = nearest_power_of_two(n)
        elif n*4 <= internalLen and internalLen > 16:
            L = nearest_power_of_two(n*2)
        else:
            # return Now
            return
        
        # keep reference of old data
        tmp = self._data
        # create new data array
        self._data = np.zeros( (L, self.ndim), dtype=np.float32 )
        # copy data
        self._data[:self._len,:] = tmp[:self._len,:]


    
    ## Indexing
    
    def __setitem__(self, index, value):
        """ Set a point or part of the pointset. """
        
        # a tuple given: set as array
        if isinstance( index, tuple):
            self.data[index] = value
        # for a slice, return subset
        elif isinstance(index, slice):
            if not is_Pointset(value):
                tmp = "When slicing with a single index "
                raise ValueError(tmp+ "the supplied value must be a pointset.")
            self.data[index] = value.data
        # otherwise, return point
        else:
            value = self._as_point(value)
#             if not is_Point(value):
#                 tmp = "When indexing with a single index "
#                 raise ValueError(tmp + "the supplied value must be a point.")
            self.data[index] = value.data
    
    
    def __getitem__(self, index):
        """ Get a point or part of the pointset. """
        
        # Single index from numpy scalar
        if isinstance(index, np.ndarray) and index.size==1:
            index = int(index)
        
        if isinstance(index, tuple):
            # Multiple indexes: return as array
            return self.data[index]
        elif isinstance(index, slice):
            # Slice: return subset
            return Pointset( self.data[index] )
        elif isinstance(index, int):
            # Single index: return point
            return Point( self.data[index] )
        else:
            # Probably some other form of subslicing
            try:
                return Pointset( self.data[index])
            except Exception:
                return Point( self.data[index])
    
    
    def __delitem__(self, index):
        """ Remove one or multiple points from the pointset. """
        
        # if tuple, not valid
        if isinstance(index, tuple):
            raise IndexError("Can only remove points using 1D slicing.")
        
        # get start/stop/step
        if isinstance(index, slice):
            start, stop, step = index.indices(self._len)
        else:
            start, stop, step = index, index+1, 1
        
        # if stepping, do it the slow way
        if step > 1:
            indices = [i for i in range(start,stop, step)]
            for i in reversed(indices):
                del self[i]
            return
            
        # move latter block forward
        tmp = self.data[stop:,:]
        self._data[start:start+len(tmp),:] = tmp
        
        # reduce length
        self._len = start + len(tmp)
        
        # should we resize?
        self._resize_if_required()

    
    ## Appending / Extending
    
    
    def append(self, *p):
        """ append(*p)
        
        Append a point to this pointset.
        If p is not an instance of the Point class,  the constructor
        of Point is called to create one from the given argument. This
        enables pp.append(x,y,z)
        
        """
        try:
            p = self._as_point(*p)
        except Exception:
            raise ValueError(str(getExceptionInstance()))
        
        # resize if we need to
        self._resize_if_required(self._len+1)
        
        # append point
        self._data[self._len,:] = p._data
        self._len += 1
    
    
    def extend(self, pp):
        """ extend(pp)
        
        Extend this pointset with another pointset, thus combining the two.
        If pp is not an instance of the Pointset class, the constructor
        of Pointset is called to create one from the given argument.
        
        """
        
        # make sure we are dealing with a pointset
        if not is_Pointset(pp):
            try:
                pp = Pointset(pp)
            except Exception:
                raise ValueError(str(getExceptionInstance()))
        
        # check whether we can append it
        if self.ndim != pp.ndim:
            raise ValueError("Can only extend pointsets of equal dimensions.")
        
        # resize the array if needed
        newLen = self._len + pp._len
        self._resize_if_required(newLen)
        
        # append new data
        i1 = self._len
        i2 = i1 + pp._len
        self._data[i1:i2,:] = pp._data[:pp._len,:]
        self._len = newLen


    def insert(self, index, *p):
        """ insert(index, *p)
        
        Insert a point at the given index.
        
        """
        
        # check index
        if index < 0:
            index = self._len + index
        if index<0 or index>self._len:
            raise IndexError("Index to insert point out of range.")
        
        # make sure p is a point
        try:
            p = self._as_point(*p)
        except Exception:
            raise ValueError(str(getExceptionInstance()))
        
        # resize if we need to
        self._resize_if_required(self._len+1)
        
        # insert point
        tmp = self._data[index:self._len,:].copy()
        self._data[index,:] = p._data
        self._data[index+1:self._len+1,:] = tmp
        self._len += 1
    
    
    def remove(self, *p):
        """ remove(*p)
        
        Remove first occurance of the given point from the list.
        Produces an error if such a point is not present.
        See also remove_all()
        
        """
        
        # make sure p is a point
        try:
            p = self._as_point(*p)
        except Exception:
            raise ValueError(str(getExceptionInstance()))
        
        # calculate mask
        mask = np.zeros((len(self),),dtype='uint8')
        for i in range(self.ndim):
            mask += self.data[:,i]==p[i]
        
        # find points given
        I, = np.where(mask==self.ndim)
        
        # produce error if not found
        if len(I) == 0:
            raise ValueError("Given point to remove was not found.")
        
        # remove first point
        del self[ I[0] ]

    
    def remove_all(self, *p):
        """ remove_all(*p)
        
        Remove all occurances of the given point. If there
        is no occurance, no action is taken.
        
        """
        
        # make sure p is a point
        try:
            p = self._as_point(*p)
        except Exception:
            raise ValueError(str(getExceptionInstance()))
        
        # calculate mask
        mask = np.zeros((len(self),),dtype='uint8')
        for i in range(self.ndim):
            mask += self.data[:,i]==p[i]
        
        # find points given
        I, = np.where(mask==self.ndim)
        
        # remove all points (backwards!)
        indices = sorted( [i for i in I] )
        for i in reversed(indices):
            del self[i]
    
    
    def pop(self, index=-1):
        """ pop(index=-1)
        
        Removes and returns a point from the pointset. Removes the last
        by default (which is more efficient than popping from anywhere else).
        
        """
        
        # check index
        index2 = index
        if index < 0:
            index2 = self._len + index
        if index2<0 or index2>self._len:
            raise IndexError("Index to insert point out of range.")
        
        # get point
        p = self[index]
        
        # remove it
        if index == -1:
            # ooh this is fast
            self._len -= 1
            self._resize_if_required()
        else:
            del self[index]
        
        # return
        return p
    
    
    def clear(self):
        """ clear()
        
        Remove all points in the pointset.
        
        """
        self.__init__(self.ndim)
    
    
    def copy(self):
        """ copy()
        
        Return a copy of this pointset.
        
        """
        return Pointset(self._data[:self._len,:])
    
    
    ## Sequence stuff
    
    
    def __len__(self):
        return self._len
    
    def __iter__(self):
        self._index = -1
        return self
    
    def __next__(self):
        self._index +=1
        if self._index >= len(self): raise StopIteration
        return self[self._index]
    
    def next(self): # Python 2.x
        return self.__next__()
    
    def contains(self, *p):
        """ contains(*p)
        
        Check whether a point is already in this set.
        
        """
        
        # make sure p is a point
        try:
            p = self._as_point(*p)
        except Exception:
            raise ValueError(str(getExceptionInstance()))
        
        mask = np.zeros((len(self),),dtype='uint8')
        for i in range(self.ndim):
            mask += self.data[:,i]==p[i]
        if mask.max() >= self.ndim:
            return True
        else:
            return False


    ## String representation
    
    def __str__(self):
        """ Return a nice string representation of the pointset. """
        s = ""
        for i in range(len(self)):
            s += "%3d: " % (i)
            s += str(self[i])
            s += "\n"
        return s

    def __repr__(self):
        """" Return short(one line) string representation of the pointset. """
        return "<pointset with %i points of %i dimensions>" % (
            len(self), self.ndim )



class Aarray(np.ndarray):
    """ Aarray(shape_or_array, sampling=None, origin=None, fill=None,
                dtype='float32', **kwargs)
    
    Anisotropic array; inherits from numpy.ndarray and adds a sampling
    and origin property which gives the sample distance and offset for
    each dimension.
    
    Parameters
    ----------
    shape_or_array : shape-tuple or numpy.ndarray
        Specifies the shape of the produced array. If an array instance is
        given, the returned Aarray is a view of the same data (i.e. no data
        is copied).
    sampling : tuple of ndim elements
        Specifies the sample distance (i.e. spacing between elements) for
        each dimension. Default is all ones.
    origin : tuple of ndim elements
        Specifies the world coordinate at the first element for each dimension.
        Default is all zeros.
    fill : scalar (optional)
        If given, and the first argument is not an existing array,
        fills the array with this given value.
    dtype : any valid numpy data type
        The type of the data
    
    All extra arguments are fed to the constructor of numpy.ndarray.
    
    Implemented properties and methods
    -----------------------------------
      * sampling - The distance between samples as a tuple
      * origin - The origin of the data as a tuple
      * get_start() - Get the origin of the data as a Point instance
      * get_end() - Get the end of the data as a Point instance
      * get_size() - Get the size of the data as a Point instance
      * sample() - Sample the value at the given point
      * point_to_index() - Given a poin, returns the index in the array
      * index_to_point() - Given an index, returns the world coordinate
    
    Slicing
    -------
    This class is aware of slicing. This means that when obtaining a part
    of the data (for exampled 'data[10:20,::2]'), the origin and sampling
    of the resulting array are set appropriately.
    
    When applying mathematical opertaions to the data, or applying
    functions that do not change the shape of the data, the sampling
    and origin are copied to the new array. If a function does change
    the shape of the data, the sampling are set to all zeros and ones
    for the origin and sampling, respectively.
    
    World coordinates vs tuples
    ---------------------------
    World coordinates are expressed as Point instances (except for the
    "origin" property). Indices as well as the "sampling" and "origin"
    attributes are expressed as tuples in z,y,x order.
    
    """
    
    _is_Aarray = True
    
    def __new__(cls, shapeOrArray, sampling=None, origin=None, fill=None,
                dtype='float32', **kwargs):
        
        if isinstance(shapeOrArray, np.ndarray):
            shape = shapeOrArray.shape
            ob = shapeOrArray.view(cls)
            if is_Aarray(shapeOrArray):
                if sampling is None:
                    sampling = shapeOrArray.sampling
                if origin is None:
                    origin = shapeOrArray.origin
        else:
            shape = shapeOrArray
            ob = np.ndarray.__new__(cls, shape, dtype=dtype, **kwargs)
            if fill is not None:
                ob.fill(fill)
        
        # init sampling and origin
        ob._sampling = tuple( [1.0 for i in ob.shape] )
        ob._origin = tuple( [0.0 for i in ob.shape] )
        
        # set them
        if sampling:
            ob.sampling = sampling
        if origin:
            ob.origin = origin
        
        # return
        return ob
    
    
    def __array_finalize__(self, ob):
        """ So the sampling and origin is maintained when doing
        calculations with the array. """
        #if hasattr(ob, '_sampling') and hasattr(ob, '_origin'):
        if isinstance(ob, Aarray):
            if self.shape == ob.shape:
                # Copy sampling and origin for math operation
                self._sampling = tuple( [i for i in ob._sampling] )
                self._origin = tuple( [i for i in ob._origin] )
            else:
                # Don't bother (__getitem__ will set them after this)
                # Other functions that change the shape cannot be trusted.
                self._sampling = tuple( [1.0 for i in self.shape] )
                self._origin = tuple( [0.0 for i in self.shape] )
        elif isinstance(self, Aarray):
            # This is an Aarray, but we do not know where it came from
            self._sampling = tuple( [1.0 for i in self.shape] )
            self._origin = tuple( [0.0 for i in self.shape] )
    
    
    def __getslice__(self, i, j):
        # Called only when indexing first dimension and without a step
        
        # Call base getitem method
        ob = np.ndarray.__getslice__(self, i, j)
        
        # Perform sampling and origin corrections
        sampling, origin = self._correct_sampling(slice(i,j))
        ob.sampling = sampling
        ob.origin = origin
        
        # Done
        return ob
    
    
    def __getitem__(self, index):
        
        # Call base getitem method
        ob = np.ndarray.__getitem__(self, index)
        
        if isinstance(index, np.ndarray):
            # Masked or arbitrary indices; sampling and origin irrelevant
            ob = np.asarray(ob)
        elif isinstance(ob, Aarray):
            # If not a scalar, perform sampling and origin corrections
            # This means there is only a very small performance penalty
            sampling, origin = self._correct_sampling(index)
            if sampling:
                ob.sampling = sampling
                ob.origin = origin
        
        # Return
        return ob
    
    
    def _correct_sampling(self, index):
        """ _correct_sampling(index)
        
        Get the new sampling and origin when slicing.
        
        """
        
        # Init origin and sampling
        _origin = self._origin
        _sampling = self._sampling
        origin = []
        sampling = []
        
        # Get index always as a tuple and complete
        index2 = [None]*len(self._sampling)
        if not isinstance(index, (list,tuple)):
            index2[0] = index
        else:
            try:
                for i in range(len(index)):
                    index2[i] = index[i]
            except Exception:
                index2[0] = index
        
        # Process
        for i in range(len(index2)):
            ind = index2[i]
            if isinstance(ind, slice):
                # print(ind.start, ind.step)
                if ind.start is None:
                    origin.append( _origin[i] )
                else:
                    origin.append( _origin[i] + ind.start*_sampling[i] )
                if ind.step is None:
                    sampling.append(_sampling[i])
                else:
                    sampling.append(_sampling[i]*ind.step)
            elif ind is None:
                origin.append( _origin[i] )
                sampling.append(_sampling[i])
            else:
                pass # singleton dimension that pops out
        
        # Return
        return sampling, origin
    
    
    def _set_sampling(self,sampling):
        if not isinstance(sampling, (list,tuple)):
            raise ValueError("Sampling must be specified as a tuple or list.")
        if len(sampling) != len(self.shape):
            raise ValueError("Sampling given must match shape.")
        for i in sampling:
            if i <= 0:
                raise ValueError("Sampling elements must be larger than zero.")
        # set
        tmp = [float(i) for i in sampling]
        self._sampling = tuple(tmp)
    
    
    def _get_sampling(self):
        l1, l2 = len(self._sampling), len(self.shape)
        if l1 < l2:
            tmp = list(self._sampling)
            tmp.extend( [1 for i in range(l2-l1)] )
            return tuple( tmp )
        elif l1 > l2:
            tmp = [self._sampling[i] for i in range(l2)]
            return tuple(tmp)
        else:
            return self._sampling
    
    sampling = property(_get_sampling, _set_sampling, None,
        "A tuple with the sample distance for each dimension.")
    
    
    def _set_origin(self,origin):
        if not isinstance(origin, (list,tuple)):
            raise ValueError("Origin must be specified as a tuple or list.")
        if len(origin) != len(self.shape):
            raise ValueError("Origin given must match shape.")
        # set
        tmp = [float(i) for i in origin]
        self._origin = tuple(tmp)
    
    
    def _get_origin(self):
        l1, l2 = len(self._origin), len(self.shape)
        if l1 < l2:
            tmp = list(self._origin)
            tmp.extend( [0 for i in range(l2-l1)] )
            return tuple( tmp )
        elif l1 > l2:
            tmp = [self._origin[i] for i in range(l2)]
            return tuple(tmp)
        else:
            return self._origin
    
    origin = property(_get_origin, _set_origin, None,
        "A tuple with the origin for each dimension.")
    
    
    def point_to_index(self, point, non_on_index_error=False):
        """ point_to_index(point, non_on_index_error=False)
        
        Given a point returns the sample index (z,y,x,..) closest
        to the given point. Returns a tuple with as many elements
        as there are dimensions.
        
        If the point is outside the array an IndexError is raised by default,
        and None is returned when non_on_index_error == True.
        
        """
        
        # check
        if not is_Point(point):
            raise ValueError("Given point must be an instance of Point.")
        if point.ndim != len(self.shape):
            raise ValueError("Given point must match the number of dimensions.")
        
        # calculate indices
        ii = []
        for i in range(point.ndim):
            s = self.shape[i]
            p = ( point[-(i+1)] - self._origin[i] ) / self._sampling[i]
            p = int(p+0.5)
            if p<0 or p>=s:
                ii = None
                break
            ii.append(p)
        
        # return
        if ii is None and non_on_index_error:
            return None
        elif ii is None:
            raise IndexError("Sample position out of range: %s" % str(point))
        else:
            return tuple(ii)
    
    
    def sample(self, point, default=None):
        """ sample(point, default=None)
        
        Take a sample of the array, given the given point
        in world-coordinates, i.e. transformed using sampling.
        By default raises an IndexError if the point is not inside
        the array, and returns the value of "default" if it is given.
        
        """
        
        tmp = self.point_to_index(point,True)
        if tmp is None:
            if default is None:
                ps = str(point)
                raise IndexError("Sample position out of range: %s" % ps)
            else:
                return default
        return self[tmp]


    def index_to_point(self, *index):
        """ index_to_point(*index)
        
        Given a multidimensional index, get the corresponding point in world
        coordinates.
        
        """
        # check
        if len(index)==1:
            index = index[0]
        if not hasattr(index,'__len__'):
            index = [index]
        if len(index) != len(self.shape):
            raise IndexError("Invalid number of indices.")
        
        # init point as list
        pp = []
        # convert
        for i in range(len(self.shape)):
            ii = index[i]
            if ii<0:
                ii = self.shape[i] - ii
            p = ii * self._sampling[i] + self._origin[i]
            pp.append(p)
        # return
        pp.reverse()
        return Point(pp)


    def get_size(self):
        """ get_size()
        
        Get the size (as a vector) of the array expressed in world coordinates.
        
        """
        pp = []
        for i in range(len(self.shape)):
            pp.append( self._sampling[i] * self.shape[i] )
        pp.reverse()
        return Point(pp)
    
    
    def get_start(self):
        """ get_start()
        
        Get the origin of the array expressed in world coordinates.
        Differs from the property 'origin' in that this method returns
        a point rather than indices z,y,x.
        
        """
        pp = [i for i in self.origin]
        pp.reverse()
        return Point(pp)
    
    
    def get_end(self):
        """ get_end()
        
        Get the end of the array expressed in world coordinates.
        
        """
        pp = []
        for i in range(len(self.shape)):
            pp.append( self._origin[i] + self._sampling[i] * self.shape[i] )
        pp.reverse()
        return Point(pp)



class Quaternion(object):
    """ Quaternion(w=1, x=0, y=0, z=0, normalize=True)
    
    A quaternion is a mathematically convenient way to
    describe rotations.
    
    """
    
    _is_Quaternion = True
    
    def __init__(self, w=1, x=0, y=0, z=0, normalize=True):
        
        self.w = float(w)
        self.x, self.y, self.z = float(x), float(y), float(z)
        if normalize:
            self._normalize()
    
    
    def __repr__(self):
        return "<Quaternion object %1.3g + %1.3gi + %1.3gj + %1.3gk>" % (
                self.w, self.x, self.y, self.z)
    
    
    def copy(self):
        """ copy()
        
        Create an exact copy of this quaternion.
        
        """
        return Quaternion(self.w, self.x, self.y, self.z, False)
    
    
    def norm(self):
        """ norm()
        
        Returns the norm of the quaternion.
        norm = w**2 + x**2 + y**2 + z**2
        
        """
        tmp = self.w**2 + self.x**2 + self.y**2 + self.z**2
        return tmp**0.5
    
    
    def _normalize(self):
        """ _normalize()
        
        Make the quaternion unit length.
        
        """
        # Get length
        L = self.norm()
        if not L:
            raise ValueError('Quaternion cannot have 0-length.')
        # Correct
        self.w /= L
        self.x /= L
        self.y /= L
        self.z /= L
    
    
    def normalize(self):
        """ normalize()
        
        Returns a normalized (unit length) version of the quaternion.
        
        """
        new = self.copy()
        new._normalize()
        return new
    
    
    def conjugate(self):
        """ conjugate()
        
        Obtain the conjugate of the quaternion.
        This is simply the same quaternion but with the sign of the
        imaginary (vector) parts reversed.
        
        """
        new = self.copy()
        new.x *= -1
        new.y *= -1
        new.z *= -1
        return new
    
    
    def inverse(self):
        """ inverse()
        returns q.conjugate()/q.norm()**2
        So if the quaternion is unit length, it is the same
        as the conjugate.
        """
        new = self.conjugate()
        tmp = self.norm()**2
        new.w /= tmp
        new.x /= tmp
        new.y /= tmp
        new.z /= tmp
        return new
    
    
    def exp(self):
        """ exp()
        
        Returns the exponent of the quaternion.
        (not tested)
        
        """
        
        # Init
        vecNorm = self.x**2 + self.y**2 + self.z**2
        wPart = np.exp(self.w)
        q = Quaternion()
        
        # Calculate
        q.w = wPart * np.cos(vecNorm)
        q.x = wPart * self.x * np.sin(vecNorm) / vecNorm
        q.y = wPart * self.y * np.sin(vecNorm) / vecNorm
        q.z = wPart * self.z * np.sin(vecNorm) / vecNorm
        
        return q
    
    
    def log(self):
        """ log()
        
        Returns the natural logarithm of the quaternion.
        (not tested)
        
        """
        
        # Init
        norm = self.norm()
        vecNorm = self.x**2 + self.y**2 + self.z**2
        tmp = self.w / norm
        q = Quaternion()
        
        # Calculate
        q.w = np.log(norm)
        q.x = np.log(norm) * self.x * np.arccos(tmp) / vecNorm
        q.y = np.log(norm) * self.y * np.arccos(tmp) / vecNorm
        q.z = np.log(norm) * self.z * np.arccos(tmp) / vecNorm
        
        return q
    
    
    def __add__(self, q):
        """ Add quaternions. """
        new = self.copy()
        new.w += q.w
        new.x += q.x
        new.y += q.y
        new.z += q.z
        return new
    
    
    def __sub__(self, q):
        """ Subtract quaternions. """
        new = self.copy()
        new.w -= q.w
        new.x -= q.x
        new.y -= q.y
        new.z -= q.z
        return new
    
    
    def __mul__(self, q2):
        """ Multiply two quaternions. """
        new = Quaternion()
        q1 = self
        new.w = q1.w*q2.w - q1.x*q2.x - q1.y*q2.y - q1.z*q2.z
        new.x = q1.w*q2.x + q1.x*q2.w + q1.y*q2.z - q1.z*q2.y
        new.y = q1.w*q2.y + q1.y*q2.w + q1.z*q2.x - q1.x*q2.z
        new.z = q1.w*q2.z + q1.z*q2.w + q1.x*q2.y - q1.y*q2.x
        return new
    
    
    def rotate_point(self, p):
        """ rotate_point(p)
        
        Rotate a Point instance using this quaternion.
        
        """
        # Prepare
        p = Quaternion(0, p.x, p.y, p.z, False) # Do not normalize!
        q1 = self.normalize()
        q2 = self.inverse()
        # Apply rotation
        r = (q1*p)*q2
        # Make point and return
        return Point(r.x, r.y, r.z)
    
    
    def get_matrix(self):
        """ get_matrix()
        
        Create a 4x4 homography matrix that represents the rotation
        of the quaternion.
        
        """
        
        # Init matrix (remember, a matrix, not an array)
        a = np.zeros((4,4), dtype=np.float32)
        w, x, y, z = self.w, self.x, self.y, self.z
        
        # First row
        a[0,0] = 1.0 -  2.0 * ( y * y + z * z )
        a[1,0] =        2.0 * ( x * y + z * w )
        a[2,0] =        2.0 * ( x * z - y * w )
        a[3,0] = 0.0
        
        # Second row
        a[0,1] =        2.0 * ( x * y - z * w )
        a[1,1] = 1.0 -  2.0 * ( x * x + z * z )
        a[2,1] =        2.0 * ( z * y + x * w )
        a[3,1] = 0.0
        
        # Third row
        a[0,2] =        2.0 * ( x * z + y * w )
        a[1,2] =        2.0 * ( y * z - x * w )
        a[2,2] = 1.0 -  2.0 * ( x * x + y * y )
        a[3,2] = 0.0
        
        # Fourth row
        a[0,3] = 0.0
        a[1,3] = 0.0
        a[2,3] = 0.0
        a[3,3] = 1.0
        
        return a
    
    
    def get_axis_angle(self):
        """ get_axis_angle()
        
        Get the axis-angle representation of the quaternion.
        (The angle is in radians)
        
        """
        
        # Init
        angle = 2 * np.arccos(self.w)
        scale = ( self.x**2 + self.y**2 + self.z**2 )**0.5
        
        # Calc axis
        if scale:
            ax = self.x / scale
            ay = self.y / scale
            az = self.z / scale
        else:
            # No rotation, so arbitrary axis
            ax, ay, az = 1, 0, 0
        
        # Return
        return angle, ax, ay, az

    
    @classmethod
    def create_from_axis_angle(cls, angle, ax, ay, az):
        """ create_from_axis_angle(angle, ax, ay, ax)
        
        Classmethod to create a quaternion from an axis-angle representation.
        (angle should be in radians).
        
        """
        angle2 = angle/2.0
        sinang2 = np.sin(angle2)
        return Quaternion( np.cos(angle2), ax*sinang2, ay*sinang2, az*sinang2 )
    
    
    @classmethod
    def create_from_euler_angles(cls, rx, ry, rz):
        """ create_from_euler_angles( rx, ry, rz)
        
        Classmethod to create a quaternion given the euler angles.
        
        """
        
        # Obtain quaternions
        qx = Quaternion( np.cos(rx/2), np.sin(rx/2), 0, 0 )
        qy = Quaternion( np.cos(ry/2), 0, np.sin(ry/2), 0 )
        qz = Quaternion( np.cos(rz/2), 0, 0, np.sin(rz/2) )
        
        # Almost done
        return qx*qy*qz



## Main

if __name__ =='__main__':
    # A small test
    
    pp = Pointset(3)

    pp.append(1,200,3)
    pp.append(-90,-3.4,0)
    pp.append(-0.0031,0.00000498,0)
    pp.append(2,3,4)
    pp.extend(pp)
    print(pp)
    pp.contains(2,3,4)
    
    print(pp-pp[0])
    print(pp.subtract(pp[0]))



# -*- coding: utf-8 -*-
# Copyright (C) 2011, Nicolas P. Rougier
# Copyright (C) 2012, Almar Klein

""" visvis.text.freetype package.

A compact wrapper that only wraps what we need to draw text. No more, no less.

This code is based on Nicolas Rougier's freetype wrapper. The ft_xxx modules
were simply copied. I only modified the __init__.py to make it more to my
liking (not using any from x import *), and to make it much more compact
by reducing it to what we need in visvis.

"""

import ctypes
import ctypes.util
from ctypes import byref
import struct

# From six.py
import sys
PY3 = sys.version_info[0] == 3
if PY3:
    string_types = str,
else:
    string_types = basestring,  # noqa

# Imports from the low level freetype wrapper
# These names I collected by parsing this module for words starting with "FT_".
from visvis.text.freetype.ft_enums import (FT_LOAD_RENDER, FT_LOAD_FORCE_AUTOHINT,  # noqa
                                           FT_KERNING_DEFAULT, FT_KERNING_UNFITTED)  # noqa
from visvis.text.freetype.ft_types import FT_Int
from visvis.text.freetype.ft_errors import FT_Exception
from visvis.text.freetype.ft_structs import (FT_Face, FT_Glyph, FT_Matrix, FT_Vector,  # noqa
                                             FT_Library, FT_GlyphSlot, FT_Size_Metrics)  # noqa


## The actual library wrapper

def _looks_lib(fname):
    """ Returns True if the given filename looks like a dynamic library.
    Based on extension, but cross-platform and more flexible.
    """
    fname = fname.lower()
    if sys.platform.startswith('win'):
        return fname.endswith('.dll')
    elif sys.platform.startswith('darwin'):
        return fname.endswith('.dylib')
    else:
        return fname.endswith('.so') or '.so.' in fname


class FreeTypeWrapper(object):
    """ Class to find and load the FreeType dll.
    """
    
    def __init__(self):
        
        self._fname = None
        self._dll = None
        self._handle = None
        
        fname = self.find_library()
        self.load_library(fname)
    
    def find_library(self):
        
        # Get Python dirs to search (shared is for Pyzo)
        import os
        py_sub_dirs = ['shared', 'lib', 'DLLs']
        py_lib_dirs = [os.path.join(sys.prefix, d) for d in py_sub_dirs]
        if hasattr(sys, 'base_prefix'):
            py_lib_dirs += [os.path.join(sys.base_prefix, d) for d in py_sub_dirs]
        py_lib_dirs = [d for d in py_lib_dirs if os.path.isdir(d)]
        for py_lib_dir in py_lib_dirs:
            for fname in os.listdir(py_lib_dir):
                if 'freetype' in fname.lower() and _looks_lib(fname):
                    return os.path.join(py_lib_dir, fname)
        
        # Search in resources (only when frozen)
        if getattr(sys, 'frozen', None):
            from visvis.core.misc import getResourceDir
            import os
            for fname in os.listdir(getResourceDir()):
                if 'freetype' in fname.lower() and _looks_lib(fname):
                    return os.path.join(getResourceDir(), fname)
        
        # Try if ctypes knows where to find the freetype library
        fname = ctypes.util.find_library('freetype')
        
        # Try harder
        if not fname:
            if sys.platform.startswith('win'):
                NBITS = 8 * struct.calcsize("P")
                fname = ctypes.util.find_library('freetype6_'+str(NBITS))
            else:
                fname = 'libfreetype.so.6'
        
        # So far so good?
        if fname:
            return fname
        else:
            raise RuntimeError('Freetype library could not be found.')
    
    def load_library(self, fname):
         
        # Try loading it
        try:
            if sys.platform.startswith('win'):
                #dll = ctypes.windll.LoadLibrary(fname)
                dll = ctypes.cdll.LoadLibrary(fname)
            else:
                dll = ctypes.CDLL(fname)
        except OSError:
            raise RuntimeError('Freetype library could not be loaded.')
        
        # Store
        self._fname, self._dll = fname, dll
    
    def get_handle(self):
        '''
        Get unique FT_Library handle
        '''
        if not self._handle:
            self._handle = FT_Library( )
            error = self._dll.FT_Init_FreeType( byref(self._handle) )
            if error: raise FT_Exception(error)
        return self._handle
    
    def del_library(self):
        if self._handle:
            # No need to try-catch; an exception in a __del__ is never shown anyway
            self._dll.FT_Done_FreeType(self._handle)
            self._handle = None
    
    def __del__(self):
        self.del_library()

    @property
    def filename(self):
        return self._fname
    
    @property
    def version(self):
        '''
        Return the version of the FreeType library being used as a tuple of
        ( major version number, minor version number, patch version number )
        '''
        amajor = FT_Int()
        aminor = FT_Int()
        apatch = FT_Int()
        library = self.get_handle()
        self._dll.FT_Library_Version(library, byref(amajor), byref(aminor), byref(apatch))
        return (amajor.value, aminor.value, apatch.value)


# Instantiate wrapper
FT = FreeTypeWrapper()

# These functions are used in this module
FT_New_Face            = FT._dll.FT_New_Face
FT_Open_Face           = FT._dll.FT_Open_Face
FT_Done_Face           = FT._dll.FT_Done_Face
FT_Set_Char_Size       = FT._dll.FT_Set_Char_Size
FT_Set_Pixel_Sizes     = FT._dll.FT_Set_Pixel_Sizes
FT_Load_Char           = FT._dll.FT_Load_Char
FT_Set_Transform       = FT._dll.FT_Set_Transform
FT_Get_Kerning         = FT._dll.FT_Get_Kerning
FT_Get_Glyph           = FT._dll.FT_Get_Glyph
FT_Get_Char_Index      = FT._dll.FT_Get_Char_Index


## Higher level classes
# The classes below were simply copied from the __init__.py of Nicolas'
# freetype wrapper. I dont need all the methods, but in this way I can
# keep up to date by simply pasting the classes in.
#
# Note that GlyphSlot.GlyphSlot is broken. We dont use it, but let's keep the
# class intact.


# Direct wrapper (simple renaming)
Vector = FT_Vector
Matrix = FT_Matrix


class GlyphSlot( object ):
    '''
    FT_GlyphSlot wrapper.

    FreeType root glyph slot class structure. A glyph slot is a container where
    individual glyphs can be loaded, be they in outline or bitmap format.
    '''

    def __init__( self, slot ):
        '''
        Create GlyphSlot object from an FT glyph slot.

        Parameters:
        -----------
          glyph: valid FT_GlyphSlot object
        '''
        self._FT_GlyphSlot = slot

    # def get_glyph( self ):
    #     '''
    #     A function used to extract a glyph image from a slot. Note that the
    #     created FT_Glyph object must be released with FT_Done_Glyph.
    #     '''
    #     aglyph = FT_Glyph()
    #     error = FT_Get_Glyph( self._FT_GlyphSlot, byref(aglyph) )
    #     if error: raise FT_Exception( error )
    #     return Glyph( aglyph )

    def _get_bitmap( self ):
        return Bitmap( self._FT_GlyphSlot.contents.bitmap )
    bitmap = property( _get_bitmap,
       doc = '''This field is used as a bitmap descriptor when the slot format
                is FT_GLYPH_FORMAT_BITMAP. Note that the address and content of
                the bitmap buffer can change between calls of FT_Load_Glyph and
                a few other functions.''')

    def _get_next( self ):
        return GlyphSlot( self._FT_GlyphSlot.contents.next )
    next = property( _get_next,
     doc = '''In some cases (like some font tools), several glyph slots per
              face object can be a good thing. As this is rare, the glyph slots
              are listed through a direct, single-linked list using its 'next'
              field.''')

    advance = property( lambda self: self._FT_GlyphSlot.contents.advance,
        doc = '''This shorthand is, depending on FT_LOAD_IGNORE_TRANSFORM, the
                 transformed advance width for the glyph (in 26.6 fractional
                 pixel format). As specified with FT_LOAD_VERTICAL_LAYOUT, it
                 uses either the 'horiAdvance' or the 'vertAdvance' value of
                 'metrics' field.''')

    # def _get_outline( self ):
    #     return Outline( self._FT_GlyphSlot.contents.outline )
    # outline = property( _get_outline,
    #     doc = '''The outline descriptor for the current glyph image if its
    #              format is FT_GLYPH_FORMAT_OUTLINE. Once a glyph is loaded,
    #              'outline' can be transformed, distorted, embolded,
    #              etc. However, it must not be freed.''')

    format = property( lambda self: self._FT_GlyphSlot.contents.format,
       doc = '''This field indicates the format of the image contained in the
                glyph slot. Typically FT_GLYPH_FORMAT_BITMAP,
                FT_GLYPH_FORMAT_OUTLINE, or FT_GLYPH_FORMAT_COMPOSITE, but
                others are possible.''')

    bitmap_top  = property( lambda self:
                             self._FT_GlyphSlot.contents.bitmap_top,
            doc = '''This is the bitmap's top bearing expressed in integer
                     pixels. Remember that this is the distance from the
                     baseline to the top-most glyph scanline, upwards y
                     coordinates being positive.''')

    bitmap_left = property( lambda self:
                            self._FT_GlyphSlot.contents.bitmap_left,
            doc = '''This is the bitmap's left bearing expressed in integer
                     pixels. Of course, this is only valid if the format is
                     FT_GLYPH_FORMAT_BITMAP.''')

    linearHoriAdvance = property( lambda self:
                                  self._FT_GlyphSlot.contents.linearHoriAdvance,
                  doc = '''The advance width of the unhinted glyph. Its value
                           is expressed in 16.16 fractional pixels, unless
                           FT_LOAD_LINEAR_DESIGN is set when loading the glyph.
                           This field can be important to perform correct
                           WYSIWYG layout. Only relevant for outline glyphs.''')

    linearVertAdvance = property( lambda self:
                                  self._FT_GlyphSlot.contents.linearVertAdvance,
                  doc = '''The advance height of the unhinted glyph. Its value
                           is expressed in 16.16 fractional pixels, unless
                           FT_LOAD_LINEAR_DESIGN is set when loading the glyph.
                           This field can be important to perform correct
                           WYSIWYG layout. Only relevant for outline glyphs.''')

class SizeMetrics( object ):
    '''
    The size metrics structure gives the metrics of a size object.

    **Note**

    The scaling values, if relevant, are determined first during a size
    changing operation. The remaining fields are then set by the driver. For
    scalable formats, they are usually set to scaled values of the
    corresponding fields in Face.

    Note that due to glyph hinting, these values might not be exact for certain
    fonts. Thus they must be treated as unreliable with an error margin of at
    least one pixel!

    Indeed, the only way to get the exact metrics is to render all glyphs. As
    this would be a definite performance hit, it is up to client applications
    to perform such computations.

    The SizeMetrics structure is valid for bitmap fonts also.
    '''

    def __init__(self, metrics ):
        '''
        Create a new SizeMetrics object.

        Parameters:
        -----------
          metrics : a FT_SizeMetrics
        '''
        self._FT_Size_Metrics = metrics

    x_ppem = property( lambda self: self._FT_Size_Metrics.x_ppem,
       doc = '''The width of the scaled EM square in pixels, hence the term
                'ppem' (pixels per EM). It is also referred to as 'nominal
                width'.''' )

    y_ppem = property( lambda self: self._FT_Size_Metrics.y_ppem,
       doc = '''The height of the scaled EM square in pixels, hence the term
                'ppem' (pixels per EM). It is also referred to as 'nominal
                height'.''' )

    x_scale = property( lambda self: self._FT_Size_Metrics.x_scale,
        doc = '''A 16.16 fractional scaling value used to convert horizontal
                 metrics from font units to 26.6 fractional pixels. Only
                 relevant for scalable font formats.''' )

    y_scale = property( lambda self: self._FT_Size_Metrics.y_scale,
        doc = '''A 16.16 fractional scaling value used to convert vertical
                 metrics from font units to 26.6 fractional pixels. Only
                 relevant for scalable font formats.''' )

    ascender = property( lambda self: self._FT_Size_Metrics.ascender,
         doc = '''The ascender in 26.6 fractional pixels. See Face for the
                  details.''' )

    descender = property( lambda self: self._FT_Size_Metrics.descender,
          doc = '''The descender in 26.6 fractional pixels. See Face for the
                    details.''' )

    height = property( lambda self: self._FT_Size_Metrics.height,
       doc = '''The height in 26.6 fractional pixels. See Face for the details.''' )

    max_advance = property(lambda self: self._FT_Size_Metrics.max_advance,
            doc = '''The maximal advance width in 26.6 fractional pixels. See
                      Face for the details.''' )


class Bitmap(object):
    '''
    FT_Bitmap wrapper

    A structure used to describe a bitmap or pixmap to the raster. Note that we
    now manage pixmaps of various depths through the 'pixel_mode' field.

    Note:
    -----
    For now, the only pixel modes supported by FreeType are mono and
    grays. However, drivers might be added in the future to support more
    'colorful' options.
    '''
    def __init__(self, bitmap):
        '''
        Create a new Bitmap object.

        Parameters:
        -----------
        bitmap : a FT_Bitmap
        '''
        self._FT_Bitmap = bitmap

    rows = property(lambda self: self._FT_Bitmap.rows,
     doc = '''The number of bitmap rows.''')

    width = property(lambda self: self._FT_Bitmap.width,
      doc = '''The number of pixels in bitmap row.''')

    pitch = property(lambda self: self._FT_Bitmap.pitch,
      doc = '''The pitch's absolute value is the number of bytes taken by one
               bitmap row, including padding. However, the pitch is positive
               when the bitmap has a 'down' flow, and negative when it has an
               'up' flow. In all cases, the pitch is an offset to add to a
               bitmap pointer in order to go down one row.

               Note that 'padding' means the alignment of a bitmap to a byte
               border, and FreeType functions normally align to the smallest
               possible integer value.

               For the B/W rasterizer, 'pitch' is always an even number.

               To change the pitch of a bitmap (say, to make it a multiple of
               4), use FT_Bitmap_Convert. Alternatively, you might use callback
               functions to directly render to the application's surface; see
               the file 'example2.py' in the tutorial for a demonstration.''')

    def _get_buffer(self):
        data = [self._FT_Bitmap.buffer[i] for i in range(self.rows*self.pitch)]
        return data
    buffer = property(_get_buffer,
       doc = '''A typeless pointer to the bitmap buffer. This value should be
                aligned on 32-bit boundaries in most cases.''')

    num_grays = property(lambda self: self._FT_Bitmap.num_grays,
          doc = '''This field is only used with FT_PIXEL_MODE_GRAY; it gives
                   the number of gray levels used in the bitmap.''')

    pixel_mode = property(lambda self: self._FT_Bitmap.pixel_mode,
           doc = '''The pixel mode, i.e., how pixel bits are stored. See
                    FT_Pixel_Mode for possible values.''')
  
    palette_mode = property(lambda self: self._FT_Bitmap.palette_mode,
             doc ='''This field is intended for paletted pixel modes; it
                     indicates how the palette is stored. Not used currently.''')

    palette = property(lambda self: self._FT_Bitmap.palette,
        doc = '''A typeless pointer to the bitmap palette; this field is
                 intended for paletted pixel modes. Not used currently.''')


class Face( object ):
    '''
    FT_Face wrapper

    FreeType root face class structure. A face object models a typeface in a
    font file.
    '''
    def __init__( self, filename, index = 0 ):
        '''
        Build a new Face

        :param str filename:
            A path to the font file.
        :param int index:
               The index of the face within the font.
               The first face has index 0.
        '''
        library = FT.get_handle( )
        face = FT_Face( )
        self._FT_Face = None
        #error = FT_New_Face( library, filename, 0, byref(face) )
        u_filename = ctypes.c_char_p(filename)
        error = FT_New_Face( library, u_filename, index, byref(face) )
        if error: raise FT_Exception( error )
        self._filename = filename
        self._index = index
        self._FT_Face = face
    
    def __del__( self ):
        '''
        Discard  face object, as well as all of its child slots and sizes.
        '''
        if self._FT_Face is not None:
            FT_Done_Face( self._FT_Face )
    
    def set_char_size( self, width=0, height=0, hres=72, vres=72 ):
        '''
        This function calls FT_Request_Size to request the nominal size (in
        points).
        
        :param float width: The nominal width, in 26.6 fractional points.
        :param float height: The nominal height, in 26.6 fractional points.
        :param float hres: The horizontal resolution in dpi.
        :param float vres: The vertical resolution in dpi.

        **Note**

        If either the character width or height is zero, it is set equal to the
        other value.

        If either the horizontal or vertical resolution is zero, it is set
        equal to the other value.

        A character width or height smaller than 1pt is set to 1pt; if both
        resolution values are zero, they are set to 72dpi.

        Don't use this function if you are using the FreeType cache API.
        '''
        error = FT_Set_Char_Size( self._FT_Face, width, height, hres, vres )
        if error: raise FT_Exception( error)
    
    def set_pixel_sizes( self, width, height ):
        '''
        This function calls FT_Request_Size to request the nominal size (in
        pixels).

        Parameters:
        -----------
        width: The nominal width, in pixels.

        height: The nominal height, in pixels.
        '''
        error = FT_Set_Pixel_Sizes( self._FT_Face, width, height )
        if error: raise FT_Exception(error)
    
    def get_char_index( self, charcode ):
        '''
        Return the glyph index of a given character code. This function uses a
        charmap object to do the mapping.

        Parameters:
        -----------
        charcode: The character code.

        Note:
        -----
        If you use FreeType to manipulate the contents of font files directly,
        be aware that the glyph index returned by this function doesn't always
        correspond to the internal indices used within the file. This is done
        to ensure that value 0 always corresponds to the 'missing glyph'.
        '''
        if isinstance(charcode, string_types):
            charcode = ord( charcode )
        return FT_Get_Char_Index( self._FT_Face, charcode )

    def set_transform( self, matrix, delta ):
        '''
        A function used to set the transformation that is applied to glyph
        images when they are loaded into a glyph slot through FT_Load_Glyph.

        Parameters:
        -----------
        matrix: A pointer to the transformation's 2x2 matrix.
                Use 0 for the identity matrix.

        delta: A pointer to the translation vector.
               Use 0 for the null vector.

        Note:
        -----
        The transformation is only applied to scalable image formats after the
        glyph has been loaded. It means that hinting is unaltered by the
        transformation and is performed on the character size given in the last
        call to FT_Set_Char_Size or FT_Set_Pixel_Sizes.

        Note that this also transforms the 'face.glyph.advance'
        field, but not the values in 'face.glyph.metrics'.
        '''
        FT_Set_Transform( self._FT_Face,
                          byref(matrix), byref(delta) )

    def load_char( self, char, flags = FT_LOAD_RENDER ):
        '''
        A function used to load a single glyph into the glyph slot of a face
        object, according to its character code.

        Parameters:
        -----------
        char: The glyph's character code, according to the current charmap used
              in the face.

        flags: A flag indicating what to load for this glyph. The FT_LOAD_XXX
               constants can be used to control the glyph loading process
               (e.g., whether the outline should be scaled, whether to load
               bitmaps or not, whether to hint the outline, etc).

        Note:
        -----
        This function simply calls FT_Get_Char_Index and FT_Load_Glyph.
        '''

        if len(char) == 1:
            char = ord(char)
        error = FT_Load_Char( self._FT_Face, char, flags )
        if error: raise FT_Exception( error )
    
    def get_kerning( self, left, right, mode = FT_KERNING_DEFAULT ):
        '''
        Return the kerning vector between two glyphs of a same face.

        Parameters:
        -----------
        left: The index of the left glyph in the kern pair.
        
        right: The index of the right glyph in the kern pair.

        mode: See FT_Kerning_Mode for more information. Determines the scale
              and dimension of the returned kerning vector.

        Note:
        -----
        Only horizontal layouts (left-to-right & right-to-left) are supported
        by this method. Other layouts, or more sophisticated kernings, are out
        of the scope of this API function -- they can be implemented through
        format-specific interfaces.
        '''
        left_glyph = self.get_char_index( left )
        right_glyph = self.get_char_index( right )
        kerning = FT_Vector(0,0)
        error = FT_Get_Kerning( self._FT_Face,
                                left_glyph, right_glyph, mode, byref(kerning) )
        if error: raise FT_Exception( error )
        return kerning

    def _get_glyph( self ):
        return GlyphSlot( self._FT_Face.contents.glyph )
    
    glyph = property( _get_glyph,
      doc = '''The face's associated glyph slot(s).''')

    def _get_size( self ):
        size = self._FT_Face.contents.size
        metrics = size.contents.metrics
        return SizeMetrics(metrics)
    size = property( _get_size,
     doc = '''The current active size for this face.''')



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module baseFigure

Defines the AxisContainer and Axes classes, as well as the Legend class used
by the Axes.

"""

import OpenGL.GL as gl

from visvis.utils.pypoints import Pointset
#
import visvis as vv
from visvis.core import base
from visvis.core.base import DRAW_NORMAL, DRAW_FAST, DRAW_SHAPE, DRAW_SCREEN
from visvis.core.misc import Property, PropWithDraw, DrawAfter
from visvis.core.misc import Range, getColor, basestring
#
from visvis.core.baseWibjects import Box, DraggableBox
from visvis.core import cameras
from visvis.core.cameras import ortho
from visvis.text import Label
from visvis.core.line import Line
from visvis.core.axises import BaseAxis, CartesianAxis, PolarAxis2D
from visvis.core.light import Light


def _Screenshot():
    """ _Screenshot()
    
    Capture the screen as a numpy array to use it later.
    Used by the object picker helper to determine which item is
    under the mouse, and by the axes to buffer its content.
    
    """
    gl.glReadBuffer(gl.GL_BACK)
    xywh = gl.glGetIntegerv(gl.GL_VIEWPORT)
    x,y,w,h = xywh[0], xywh[1], xywh[2], xywh[3]
    # use floats to prevent strides etc. uint8 caused crash on qt backend.
    im = gl.glReadPixels(x, y, w, h, gl.GL_RGB, gl.GL_FLOAT)
    # reshape, flip, and store
    im.shape = h,w,3
    return im


class _BaseFigure(base.Wibject):
    """ Abstract class that the BaseFigure inherits from. It solves
    the mutual dependence of the Axes and BaseFigure classes.
    """
    pass


class AxesContainer(Box):
    """ AxesContainer(parent)
    
    A simple container wibject class to contain one Axes instance.
    Each Axes in contained in an AxesContainer instance. By default
    the axes position is expressed in pixel coordinates, while the
    container's position is expressed in unit coordinates. This
    enables advanced positioning of the Axes.
    
    When there is one axes in a figure, the container position will
    be "0,0,1,1". For subplots however, the containers are positioned
    to devide the figure in equal parts. The Axes instances themselves
    are positioned in pixels, such that when resizing, the margins for
    the tickmarks and labels remains equal.
    
    The only correct way to create (and obtain a reference to)
    an AxesContainer instance is to use:
      * axes = vv.Axes(figure)
      * container = axes.parent
    
    This container is automatically destroyed once the axes is removed.
    You can attach wibjects to an instance of this class, but note that
    the container object is destroyed as soon as the axes is gone.
    
    """
    
    def __init__(self, parent, *args, **kwargs):
        
        # check that the parent is a Figure
        if not isinstance(parent, _BaseFigure):
            raise Exception("The given parent for an AxesContainer " +
                            "should be a Figure.")
        
        # Init box
        Box.__init__(self, parent, *args, **kwargs)
        
        # Init position
        self.position = 0,0,1,1
        
        # Set properties
        self.edgeWidth = 0
        self.bgcolor = None
    
    def GetAxes(self):
        """ GetAxes()
        
        Get the axes. Creates a new axes object if it has none.
        
        """
        if self._children:
            child = self._children[0]
            if isinstance(child, Axes):
                return child
        return None
    
    
    def _DrawTree(self, mode, *args, **kwargs):
        """ _DrawTree(mode, *args, **kwargs)
        
        Pass on, but Destroy itself if axes is gone.
        
        """
        axes = self.GetAxes()
        if axes:
            # Draw normally
            base.Wibject._DrawTree(self, mode, *args, **kwargs)
        else:
            self.Destroy()
    

class Axes(base.Wibject):
    """ Axes(parent, axisClass=None)
    
    An Axes instance represents the scene with a local coordinate system
    in which wobjects can be drawn. It has various properties to influence
    the appearance of the scene, such as aspect ratio and lighting.
    
    To set the appearance of the axis (the thing that indicates x, y and z),
    use the properties of the Axis instance. For example:
    Axes.axis.showGrid = True
    
    The cameraType determines how the data is visualized and how the user
    can interact with the data.
    
    The daspect property represents the aspect ratio of the data as a
    three element tuple. The sign of the elements indicate dimensions
    being flipped. (The function imshow() for example flips the
    y-dimension). If daspectAuto is False, all dimensions are always
    equally zoomed (The function imshow() sets this to False).
    
    An Axes can be created with the function vv.subplot() or vv.gca().
    
    """
    
    def __init__(self, parent, axisClass=None):
        
        # check that the parent is a Figure or AxesContainer
        if isinstance(parent, AxesContainer):
            figure = parent.parent
        elif isinstance(parent, _BaseFigure):
            figure = parent
            parent = AxesContainer(figure)
        else:
            raise Exception("The given parent for an Axes " +
                            "should be a Figure or AxesContainer.")
        
        # call base __init__
        base.Wibject.__init__(self, parent)
        
        # objects in the scene. The Axes is the only wibject that
        # can contain wobjects. Basically, the Axes is the root
        # for all the wobjects in it.
        self._wobjects = []
        
        # data aspect ratio. If daspectAuto is True, the values
        # of daspect are ignored (only the sign is taken into account)
        self._daspect = (1.0,1.0,1.0)
        self._daspectAuto = None # None is like False, but means not being set
                
        # screenshot buffer and variable to indicate whether we can use it
        self._screenshot = None
        self._isdirty = True
        self._motionBlur = 0.0
        self._useBuffer = True
        
        # varialble to keep track of the position correction to fit labels
        self._xCorr, self._yCorr = 0, 0
        
        # create cameras and select 3D as the defaule
        self._cameras = {}
        self.camera = cameras.TwoDCamera()
        self.camera = cameras.ThreeDCamera()
        self.camera = cameras.FlyCamera()
        self.camera = '3D' # Select
        
        # init the background color of this axes
        self.bgcolor = 1,1,1  # remember that bgcolor is a property
        self.bgcolors = None
        
        # bind to event (no need to unbind because it's our own)
        self.eventMouseDown.Bind(self._OnMouseDown)
        self.eventKeyDown.Bind(self._OnKeyDown)
        self.eventScroll.Bind(self._OnScroll)
        
        # Store axis class and instantiate it
        if axisClass is None or not isinstance(axisClass, BaseAxis):
            axisClass = CartesianAxis
        self._axisClass = axisClass
        axisClass(self) # is a wobject
        
        # Let there be lights
        self._lights = []
        for i in range(8):
            self._lights.append(Light(self, i))
        # Init default light
        self.light0.On()
        
        # make current
        figure.currentAxes = self
    
    
    ## Define more methods
    
    @DrawAfter
    def SetLimits(self, rangeX=None, rangeY=None, rangeZ=None, margin=0.02):
        """ SetLimits(rangeX=None, rangeY=None, rangeZ=None, margin=0.02)
        
        Set the limits of the scene. For the 2D camera, these are taken
        as hints to set the camera view. For the 3D camear, they determine
        where the axis is drawn.
        
        Returns a 3-element tuple of visvis.Range objects.
        
        Parameters
        ----------
        rangeX : (min, max), optional
            The range for the x dimension.
        rangeY : (min, max), optional
            The range for the y dimension.
        rangeZ : (min, max), optional
            The range for the z dimension.
        margin : scalar
            Represents the fraction of the range to add for the
            ranges that are automatically obtained (default 2%).
        
        Notes
        -----
        Each range can be None, a 2 element iterable, or a visvis.Range
        object. If a range is None, the range is automatically obtained
        from the wobjects currently in the scene. To set the range that
        will fit all wobjects, simply use "SetLimits()"
        
        """
        
        # Check margin
        if margin and not isinstance(margin, float):
            raise ValueError('In SetLimits(): margin should be a float.')
        
        # if tuples, convert to ranges
        if rangeX is None or isinstance(rangeX, Range):
            pass # ok
        elif hasattr(rangeX,'__len__') and len(rangeX)==2:
            rangeX = Range(rangeX[0], rangeX[1])
        else:
            raise ValueError("Limits should be Ranges or two-element iterables.")
        if rangeY is None or isinstance(rangeY, Range):
            pass # ok
        elif hasattr(rangeY,'__len__') and len(rangeY)==2:
            rangeY = Range(rangeY[0], rangeY[1])
        else:
            raise ValueError("Limits should be Ranges or two-element iterables.")
        if rangeZ is None or isinstance(rangeZ, Range):
            pass # ok
        elif hasattr(rangeZ,'__len__') and len(rangeZ)==2:
            rangeZ = Range(rangeZ[0], rangeZ[1])
        else:
            raise ValueError("Limits should be Ranges or two-element iterables.")
        
        rX, rY, rZ = rangeX, rangeY, rangeZ
        
        if None in [rX, rY, rZ]:
            
            # find outmost range
            wobjects = self.FindObjects(base.Wobject)
            for ob in wobjects:
                
                # Ask object what it's limits are
                tmp = ob._GetLimits()
                if not tmp:
                    continue
                tmpX, tmpY, tmpZ = tmp
                
                # Check for NaNs
                if tmpX.min*0 != 0 or tmpX.max*0 != 0:
                    tmpX = None
                if tmpY.min*0 != 0 or tmpY.max*0 != 0:
                    tmpY = None
                if tmpZ.min*0 != 0 or tmpZ.max*0 != 0:
                    tmpZ = None
                
                # update min/max
                if rangeX:
                    pass
                elif tmpX and rX:
                    rX = Range( min(rX.min, tmpX.min), max(rX.max, tmpX.max) )
                elif tmpX:
                    rX = tmpX
                if rangeY:
                    pass
                elif tmpY and rY:
                    rY = Range( min(rY.min, tmpY.min), max(rY.max, tmpY.max) )
                elif tmpY:
                    rY = tmpY
                if rangeZ:
                    pass
                elif tmpZ and rZ:
                    rZ = Range( min(rZ.min, tmpZ.min), max(rZ.max, tmpZ.max) )
                elif tmpX:
                    rZ = tmpZ
        
        # default values
        if rX is None:
            rX = Range(-1,1)
        if rY is None:
            rY = Range(0,1)
        if rZ is None:
            rZ = Range(0,1)
        
        # apply margins
        if margin:
            if rangeX is None:
                tmp = rX.range * margin
                if tmp == 0: tmp = margin
                rX = Range( rX.min-tmp, rX.max+tmp )
            if rangeY is None:
                tmp = rY.range * margin
                if tmp == 0: tmp = margin
                rY = Range( rY.min-tmp, rY.max+tmp )
            if rangeZ is None:
                tmp = rZ.range * margin
                if tmp == 0: tmp = margin
                rZ = Range( rZ.min-tmp, rZ.max+tmp )
        
        # apply to each camera
        for cam in self._cameras.values():
            cam.SetLimits(rX, rY, rZ)
        
        # return
        return rX, rY, rZ
    
    
    def GetLimits(self):
        """ GetLimits()
        
        Get the limits of the axes as currently displayed. This can differ
        from what was set by SetLimits if the daspectAuto is False.  With
        a 2D camera, this returns the limits for x and y determined by the
        view.  With a 3D camera, this returns the x, y, and z extents of
        the coordinate axes.
        
        """
        return self.camera.GetLimits()
    
    
    def GetView(self):
        """ GetView()
        
        Get a dictionary with the camera parameters. The parameters are
        named so they can be changed in a natural way and fed back using
        SetView(). Note that the parameters can differ for different camera
        types.
        
        """
        return self.camera.GetViewParams()
    
    
    @DrawAfter
    def SetView(self, s=None, **kw):
        """ SetView(s=None, **kw)
        
        Set the camera view using the given dictionary with camera parameters.
        Camera parameters can also be passed as keyword/value pairs; these will
        supersede the values of the same key in s.  If neither s nor any keywords
        are set, the camera is reset to its initial state.
        
        """
        if s or kw:
            self.camera.SetViewParams(s, **kw)
        else:
            self.camera.Reset()
    
    
    def Draw(self, fast=False):
        """ Draw(fast=False)
        
        Calls Draw(fast) on its figure, as the total opengl canvas
        has to be redrawn. This might change in the future though.
        
        """
        
        if self._isbeingdrawn:
            return False
        else:
            # Make dirty
            self._isdirty = True
            
            # Draw figure
            figure = self.GetFigure()
            if figure:
                figure.Draw(fast)
            
            # Done
            return True
    
    
    @DrawAfter
    def Clear(self, clearForDestruction=False):
        """ Clear()
        
        Clear the axes. Removing all wobjects in the scene.
        
        """
        # Remove wobjects
        for w in self.wobjects:
            if isinstance(w, BaseAxis) and not clearForDestruction:
                continue
            elif hasattr(w,'Destroy'):
                w.Destroy()
    
    @property
    def wobjects(self):
        """ Get a shallow copy of the list of wobjects in the scene.
        """
        return [child for child in self._wobjects]
    
    
    def _CorrectPositionForLabels(self):
        """ _CorrectPositionForLabels()
        
        Correct the position for the labels and title etc.
        
        """
        
        # init correction
        xCorr, yCorr = 0, 0
        
        # correction should be applied for 2D camera and a valid label
        if isinstance(self.camera, cameras.TwoDCamera):
            axis = self.axis
            if isinstance(axis, PolarAxis2D):
                if axis.visible and axis.xLabel:
                    yCorr += 25
            else:
                if axis.visible:
                    yCorr += 20
                    xCorr += 60 # there's already a margin of 10 by default
                    if axis.xLabel:
                        yCorr += 20
                    if axis.yLabel:
                        xCorr += 20
        
        # check the difference
        if xCorr != self._xCorr or yCorr != self._yCorr:
            dx = self._xCorr - xCorr
            dy = self._yCorr - yCorr
            self._xCorr, self._yCorr = xCorr, yCorr
            # apply
            self.position.Correct(-dx, 0, dx, dy)
    
    ## Define more properties
    
    
    @PropWithDraw
    def bgcolors():
        """ Get/Set the colors for the axes background gradient. If used, this
        value overrides the normal bgcolor property. Notes:
          * Set to None to disable the gradient
          * Setting two colors defines a gradient from top to bottom.
          * Setting four colors sets the colors at the four corners.
          * The value must be an iterable (2 or 4 elements) in which each
            element can be converted to a color.
        """
        def fget(self):
            return self._bgcolors
        def fset(self, value):
            # None?
            if value is None:
                self._bgcolors = None
                return
            # Check
            try:
                if len(value) not in [2,4]:
                    raise ValueError('bgcolors must have 2 or 4 elements.')
            except Exception:
                # not an iterable
                raise ValueError('bgcolors must be None, or tuple/list/string.')
            # Apply
            colors = [getColor(val, 'setting bgcolors') for val in value]
            self._bgcolors = tuple(colors)
        return locals()
    
    
    @property
    def axis(self):
        """ Get the axis object associated with this axes.
        A new instance is created if it does not yet exist. This object
        can be used to change the appearance of the axis (tickmarks, labels,
        grid, etc.).
        
        See also the [[cls_BaseAxis Axis class]].
        
        """
        # Find object in root
        for object in self._wobjects:
            if isinstance(object, BaseAxis):
                return object
        else:
            # Create new and return
            return self._axisClass(self)
    
    
    @PropWithDraw
    def axisType():
        """ Get/Set the axis type to use.
        
        Currently supported are:
          * 'cartesian' - a normal axis (default)
          * 'polar' - a polar axis.
        """
        def fget(self):
            D = {PolarAxis2D:'polar', CartesianAxis:'cartesian'}
            if self._axisClass in D:
                return D[self._axisClass]
            else:
                return ''
        def fset(self, axisClass):
            # Handle string argument
            if not isinstance(axisClass, BaseAxis):
                D = {'polar':PolarAxis2D, 'cartesian':CartesianAxis}
                if axisClass not in D:
                    raise ValueError('Invalid axis class.')
                axisClass = D[axisClass.lower()]
            if axisClass is not self._axisClass:
                # Store class
                self._axisClass = axisClass
                # Remove previous
                axisList = self.FindObjects(BaseAxis)
                for axis in axisList:
                    axis.Destroy()
                # Add new
                axisClass(self)
        return locals()
    
    
    @PropWithDraw
    def camera():
        """ Get/Set the current camera.
        
        Setting can be done using:
          * The index of the camera; 1,2,3 for fly, 2d and 3d respectively.
          * A value as in the 'cameraType' property.
          * A new camera instance. This will replace any existing camera
            of the same type. To have multiple 3D cameras at the same axes,
            one needs to subclass cameras.ThreeDCamera.
        
        Shared cameras
        --------------
        One can set the camera to the camera of another Axes, so that they
        share the same camera. A camera that is shared uses daspectAuto
        property of the first axes it was attached to.
        
        Interactively changing a camera
        -------------------------------
        By default, the camera can be changed using the keyboard using the
        shortcut ALT+i, where i is the camera number. Similarly
        the daspectAuto propert can be switched with ALT+d.
        """
        def fget(self):
            return self._camera
        def fset(self, value):
            if isinstance(value, (basestring, int)):
                # Type
                self.cameraType = value
            else:
                # It must be a camera
                camera = value
                # Check
                if not isinstance(camera, cameras.BaseCamera):
                    raise ValueError('Given argument is not a camera.')
                # Store camera
                camType = camera.__class__.__name__
                oldCamera = self._cameras.get(camType, None)
                self._cameras[camType] = camera
                # Register at camera, unregister at old one
                camera._RegisterAxes(self)
                if oldCamera and oldCamera is not camera:
                    oldCamera._UnregisterAxes(self)
                # Make current and set limits
                self._camera = camera
                self.SetLimits()
        return locals()
    
    
    @PropWithDraw
    def cameraType():
        """ Get/Set the camera type to use.
        
        Currently supported are:
          * '2d' or 2  - two dimensional camera that looks down the z-dimension.
          * '3d' or 3  - three dimensional camera.
          * 'fly' or 1 - a camera like a flight sim.
        
        """
        def fget(self):
            return self._camera._NAMES[0]
        def fset(self, name):
            # Case insensitive
            if isinstance(name, basestring):
                name = name.lower()
            # Get camera with that name
            theCamera = None
            for camera in self._cameras.values():
                if name in camera._NAMES:
                    theCamera = camera
                    break
            # Set or raise error
            if theCamera:
                self._camera = theCamera
            else:
                raise ValueError("Unknown camera type!")
        return locals()
    
    
    @property
    def mousepos(self):
        """ Get position of mouse in screen pixels, relative to this axes.
        """
        figure = self.GetFigure()
        if not figure:
            return 0,0
        x,y = figure.mousepos
        pos = self.position
        return x-pos.absLeft, y-pos.absTop
    
    
    @PropWithDraw
    def daspect():
        """ Get/set the data aspect ratio of the current camera. Setting will
        also update daspect for the other cameras.
        
        The daspect is a 3-element tuple (x,y,z). If a 2-element tuple is
        given, z is assumed 1. Note that only the ratio between the values
        matters (i.e. (1,1,1) equals (2,2,2)). When a value is negative, the
        corresponding dimension is flipped.
        
        Note that if daspectAuto is True, the camera automatically changes
        its daspect to nicely scale the data to fit the screen (but the sign
        is preserved).
        """
        def fget(self):
            return self.camera.daspect
        def fset(self, value):
            # Set on all cameras
            camera = None
            for camera in self._cameras.values():
                camera.daspect = value
            # Set on self so new cameras can see what the user set.
            # Use camera's daspect, in case a 2-element tuple was used.
            if camera is not None:
                self._daspect = camera.daspect
        return locals()
    
    @property
    def daspectNormalized(self):
        """ Get the data aspect ratio, normalized such that the x scaling
        is +/- 1.
        """
        return self.camera.daspectNormalized
    
    @PropWithDraw
    def daspectAuto():
        """ Get/Set whether to scale the dimensions independently.
        
        If True, the camera changes the value of its daspect to nicely fit
        the data on screen (but the sign is preserved). This can happen
        (depending on the type of camera) during resetting, zooming, and
        resizing of the axes.
        
        If set to False, the daspect of all cameras is reverted to
        the user-set daspect.
        """
        def fget(self):
            return self._daspectAuto
        def fset(self, value):
            # Set dastecpAuto
            self._daspectAuto = bool(value)
            # Update daspect if False
            if not value:
                self.daspect = self._daspect
        return locals()
    
    
    @PropWithDraw
    def legend():
        """ Get/Set the string labels for the legend. Upon setting,
        a legend wibject is automatically shown.
        """
        def fget(self):
            return self.legendWibject._stringList
        def fset(self, value):
            self.legendWibject.SetStrings(value)
        return locals()
    
    @property
    def legendWibject(self):
        """ Get the legend wibject, so for exampe its position
        can be changed programatically.
        """
        legendWibjects = self.FindObjects(Legend)
        if not legendWibjects:
            legendWibjects = [Legend(self)] # create legend object
        return legendWibjects[-1]
    
    
    @property
    def light0(self):
        """ Get the default light source in the scene.
        """
        return self._lights[0]
    
    @property
    def lights(self):
        """ Get a list of all available lights in the scene. Only light0 is
        enabeled by default.
        """
        return [light for light in self._lights]
        
    
    @PropWithDraw
    def useBuffer():
        """ Get/Set whether to use a buffer; after drawing, a screenshot
        of the result is obtained and stored. When the axes needs to
        be redrawn, but has not changed, the buffer can be used to
        draw the contents at great speed (default True).
        """
        def fget(self):
            return self._useBuffer
        def fset(self, value):
            self._useBuffer = bool(value)
        return locals()
    
    
    @Property
    def motionBlur():
        """ Get/Set the amount of motion blur when interacting with
        this axes. The value should be a number between 0 and 1.
        Note: this is a rather useless feature :)
        """
        def fget(self):
            return self._motionBlur
        def fset(self, value):
            tmp = float(value)
            self._motionBlur = min(max(tmp,0.0),1.0)
        return locals()
    
    ## Implement methods
    
    def OnDestroy(self):
        # Clean up.
        base.Wibject.OnDestroy(self)
        self.Clear(True)
        self._camera = None
        self._cameras = {}
        # container is destroyed as soon as it notices the axes is gone
        # any wibjects are destoyed automatically by the Destroy command.
    
    
    def OnDrawShape(self, clr):
        
        # Correct size for labels (shape is the first draw pass)
        self._CorrectPositionForLabels()
        
        # Get picker helper and draw
        pickerHelper = self.GetFigure()._pickerHelper
        
        # Size of figure ...
        fig = self.GetFigure()
        w,h = fig.position.size
        
        # Find actual position in pixels, do not allow negative values
        pos = self.position.InPixels()
        pos._w, pos._h = max(pos.w, 1), max(pos.h, 1)
        pos.h_fig = h
        pos._Update()
        
        # Set viewport (note that OpenGL has origin in lower-left, visvis
        # in upper-left)
        fig._dpi_aware_viewport(pos.absLeft, h-pos.absBottom, pos.w, pos.h)
        
        self._OnDrawContent(DRAW_SHAPE, clr, pos, pickerHelper)
        
        # Prepare for wibject children (draw in full viewport)
        fig._dpi_aware_viewport(0, 0, w, h)
        gl.glDisable(gl.GL_DEPTH_TEST)
        gl.glMatrixMode(gl.GL_PROJECTION)
        gl.glLoadIdentity()
        fig._normal_ortho(0, w, h, 0)
        gl.glMatrixMode(gl.GL_MODELVIEW)
        gl.glLoadIdentity()
        
        # Transform
        self.parent._Transform() # Container
        self._Transform() # Self
    
    
    def OnDrawFast(self):
        self._OnDrawInMode(DRAW_FAST, self.bgcolor)
    
    def OnDraw(self):
        self._OnDrawInMode(DRAW_NORMAL, self.bgcolor)
    
    
    def _OnDrawInMode(self, mode, bgcolor, pickerHelper=None):
        # Draw the background of the axes and the wobjects in it.
        
        # Prepare
        if True:
            
            # Get size of figure ...
            fig = self.GetFigure()
            w,h = fig.position.size
            
            # Correction of size for labels is normally done in OnDrawShape,
            # but this is not called if user interaction is disabled ...
            if not fig.enableUserInteraction:
                self._CorrectPositionForLabels()
            
            # Find actual position in pixels, do not allow negative values
            pos = self.position.InPixels()
            pos._w, pos._h = max(pos.w, 1), max(pos.h, 1)
            pos.h_fig = h
            pos._Update()
            
            # Set viewport (note that OpenGL has origin in lower-left, visvis
            # in upper-left)
            fig._dpi_aware_viewport(pos.absLeft, h-pos.absBottom, pos.w, pos.h)
            
            # Select screenshot
            sshot = self._screenshot
        
        
        # Perform tests
        # Only if enabled on axes and if user interaction is enabled for the figure
        if self._useBuffer and fig.enableUserInteraction:
            
            # Test if we can use the screenshot
            canUseScreenshot = (    (sshot is not None) and
                                    sshot.shape[0] == pos.h and
                                    sshot.shape[1] == pos.w )
            
            # Test if we want to blur with the screenshot
            blurWithScreenshot = (  bool(self._motionBlur) and
                                    self._isdirty and
                                    mode==DRAW_FAST )
            
            # Test whether we should use the screenshot
            shouldUseScreenshot = ( canUseScreenshot and
                                    (not self._isdirty or blurWithScreenshot) )
        
        else:
            # Old school mode
            shouldUseScreenshot = False
            blurWithScreenshot = False
        
        
        # Draw content of axes (if we need to)
        if (not shouldUseScreenshot) or blurWithScreenshot:
            
            # Draw fresh
            self._OnDrawContent(mode, bgcolor, pos, pickerHelper)
            
            # Make screenshot and store/combine
            if self._useBuffer and fig.enableUserInteraction:
                tmp = _Screenshot()
                shapesMatch = (sshot is not None) and tmp.shape == sshot.shape
                if blurWithScreenshot and shapesMatch:
                    f = self._motionBlur
                    sshot[:] = f*sshot + (1.0-f)*tmp
                else:
                    self._screenshot = tmp
        
        
        # Draw screenshot (if we should)
        if shouldUseScreenshot:
            
            # Set view
            gl.glMatrixMode(gl.GL_PROJECTION)
            gl.glLoadIdentity()
            ortho( 0, 1, 0, 1)
            gl.glMatrixMode(gl.GL_MODELVIEW)
            gl.glLoadIdentity()
            
            # Apply bitmap directly
            sshot = self._screenshot
            gl.glRasterPos(0,0)
            gl.glDrawPixels(pos.w, pos.h, gl.GL_RGB, gl.GL_FLOAT, sshot)
        
        
        # # Set viewport to the full figure and disable depth test
        if True:
            fig._dpi_aware_viewport(0, 0, w, h)
            gl.glDisable(gl.GL_DEPTH_TEST)
        
        
        # Draw axis if using the 2D camera
        if isinstance(self.camera, cameras.TwoDCamera):
            # Let axis object for 2D-camera draw in screen coordinates
            # in the full viewport.
            # Note that if the buffered screenshot is used and the content
            # is not drawn, the axis' OnDraw method is not called, and the
            # ticks are therefore not re-calculated (which is time-consuming).
            
            # Set view
            gl.glMatrixMode(gl.GL_PROJECTION)
            gl.glLoadIdentity()
            fig._dpi_aware_ortho( 0, w, 0, h)  # Note that 0 and h are swapped
            gl.glMatrixMode(gl.GL_MODELVIEW)
            gl.glLoadIdentity()
            
            # Draw
            for item in self._wobjects:
                if isinstance(item, BaseAxis):
                    item._DrawTree(DRAW_SCREEN)
        
        
        # Prepare for drawing child wibjects in screen coordinates
        if True:
            
            # Set view
            gl.glMatrixMode(gl.GL_PROJECTION)
            gl.glLoadIdentity()
            fig._normal_ortho(0, w, h, 0)
            gl.glMatrixMode(gl.GL_MODELVIEW)
            gl.glLoadIdentity()
            
            
            # Transform
            self.parent._Transform() # Container
            self._Transform() # Self
        
        
        # We're clean now ...
        if mode != DRAW_SHAPE:
            self._isdirty = False
    
    
    def _OnDrawContent(self, mode, bgcolor, pos, pickerHelper=None):
        
        # Draw background
        if bgcolor:
            
            # Set view
            gl.glMatrixMode(gl.GL_PROJECTION)
            gl.glLoadIdentity()
            ortho( 0, 1, 0, 1)
            gl.glMatrixMode(gl.GL_MODELVIEW)
            gl.glLoadIdentity()
            
            # Overwrite all
            gl.glDisable(gl.GL_DEPTH_TEST)
            
            # Maybe this would be better (set initial alpha to 0). But things
            # work without it and I'm too afraid to break things.
            # gl.glBlendFuncSeparate(gl.GL_ONE, gl.GL_ZERO, gl.GL_ZERO, gl.GL_ZERO)
            
            # Define colors, use gradient?
            bgcolor1 = bgcolor2 = bgcolor3 = bgcolor4 = bgcolor
            if mode != DRAW_SHAPE and self.bgcolors:
                gl.glShadeModel(gl.GL_SMOOTH)
                if len(self.bgcolors) == 2:
                    bgcolor1 = bgcolor2 = self.bgcolors[0]
                    bgcolor3 = bgcolor4 = self.bgcolors[1]
                elif len(self.bgcolors) == 4:
                    bgcolor1, bgcolor2, bgcolor3, bgcolor4 = self.bgcolors
            
            # Draw
            gl.glBegin(gl.GL_POLYGON)
            gl.glColor3f(bgcolor3[0], bgcolor3[1], bgcolor3[2])
            gl.glVertex2f(0,0)
            gl.glColor3f(bgcolor1[0], bgcolor1[1], bgcolor1[2])
            gl.glVertex2f(0,1)
            gl.glColor3f(bgcolor2[0], bgcolor2[1], bgcolor2[2])
            gl.glVertex2f(1,1)
            gl.glColor3f(bgcolor4[0], bgcolor4[1], bgcolor4[2])
            gl.glVertex2f(1,0)
            gl.glEnd()
            
            # Reset
            gl.glEnable(gl.GL_DEPTH_TEST)
            # gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
            
        # Draw items in world coordinates
        if True:
            
            # Setup the camera
            self.camera.SetView()
            
            # Draw stuff, but wait with lines
            lines2draw = []
            for item in self._wobjects:
                if isinstance(item, (Line, BaseAxis)):
                    lines2draw.append(item)
                else:
                    item._DrawTree(mode, pickerHelper)
            
            # Lines (and the axis) are a special case. In order to blend
            # them well, we should draw textures, meshes etc, first.
            # Note that this does not work if lines textures are children
            # of each-other. in that case they should be added to the scene
            # in the correct order.
            for item in lines2draw:
                item._DrawTree(mode, pickerHelper)
        
        # Draw items in screen coordinates
        if mode != DRAW_SHAPE:
            
            pixelRatio = self.GetFigure()._devicePixelRatio
            
            # Set camera to screen coordinates.
            gl.glMatrixMode(gl.GL_PROJECTION)
            gl.glLoadIdentity()
            h = pos.h_fig
            ortho(pos.absLeft * pixelRatio,
                  pos.absRight * pixelRatio,
                  (h - pos.absBottom) * pixelRatio,
                  (h - pos.absTop) * pixelRatio,
            )
            gl.glMatrixMode(gl.GL_MODELVIEW)
            gl.glLoadIdentity()
            
            # Allow wobjects to draw in screen coordinates
            # Note that the axis for the 2d camera needs to draw beyond
            # the viewport of the axes, and is therefore drawn later.
            gl.glEnable(gl.GL_DEPTH_TEST)
            is2dcam = isinstance(self.camera, cameras.TwoDCamera)
            for item in self._wobjects:
                if is2dcam and isinstance(item, BaseAxis):
                    continue
                item._DrawTree(DRAW_SCREEN)
    
    
    def _OnMouseDown(self, event):
        self.MakeCurrent()
    
    def _OnScroll(self, event):
        SCROLL_ZOOM_FACTOR = 1.1
        self.camera.zoom *= SCROLL_ZOOM_FACTOR**event.verticalSteps
    
    def _OnKeyDown(self, event):
        """ Give user a lot of control via special keyboard input.
        Kind of a secret function, as not all keys are not documented.
        """
        
        # Only do this if this is the current axes
        f = self.GetFigure()
        if not (f and self is f.currentAxes):
            return False
        
        if vv.KEY_ALT in event.modifiers and len(event.modifiers)==1:
            
            numbers = [ord(i) for i in '0123456789']
            
            if event.key in numbers:
                self.cameraType = int(chr(event.key))
            elif event.key == ord('d'):
                self.daspectAuto = not self.daspectAuto
            elif event.key == ord('a'):
                self.axis.visible = not self.axis.visible
            elif event.key == ord('g'):
                self.axis.showGrid = not any(self.axis.showGrid)
            elif event.key == ord('b'):
                if self.bgcolor == (1,1,1):
                    self.bgcolor = 'k'
                    self.axis.axisColor = 'w'
                else:
                    self.bgcolor = 'w'
                    self.axis.axisColor = 'k'
            else:
                return False
        else:
            return False
    
    
    def MakeCurrent(self):
        """ MakeCurrent()
        
        Make this the current axes. Also makes the containing figure
        the current figure.
        
        """
        f = self.GetFigure()
        if f:
            f.currentAxes = self
            f.MakeCurrent()


class Legend(DraggableBox):
    """ Legend(parent)
    
    A legend is a wibject that should be a child (does not have
    to be the direct child) of an axes. It displays a description for
    each line in the axes, and is draggable.
    
    A Legend can be shown with the function vv.legend(), or using the
    Axes.legend property.
    
    """
    
    def __init__(self, parent):
        DraggableBox.__init__(self, parent)
        
        # params for the layout
        self._linelen = 40
        self._xoffset = 10
        self._yoffset = 3
        self._yspacing = 16
        
        # position in upper left by default
        self.position = 10, 10
        self.bgcolor = 'w'
        
        # start with nothing
        self._stringList = []
        self.visible = False
        
        # by creating a _wobjects attribute, we are allowed to hold
        # wobjects, but our ourselves responsible for drawing them
        self._wobjects = []
    
    
    def _AddLineAndLabel(self, text, yspacing=1.0, twoPoints=True):
        """ Add a line and label to our pool. """
        # get y position
        index = len(self._wobjects)
        y = self._yoffset + yspacing * (index)
        # create label
        label = Label(self, text)
        label.bgcolor = ''
        label.position = self._xoffset*2 + twoPoints*self._linelen, y
        deltax, deltay = label.GetVertexLimits()
        #y2 = label.position.h / 2
        y2 = (deltay[1] + deltay[0]) / 2
        # create 2-element pointset
        pp = Pointset(2)
        pp.append(self._xoffset, y + y2)
        if twoPoints:
            pp.append(self._xoffset + self._linelen, y + y2)
        # create line
        line = Line(self, pp) # line has no parent
        # return
        return line, label
    
    
    def SetStrings(self, *stringList):
        """ SetStrings(*stringList)
        
        Set the strings of the legend labels.
        
        """
        # Note that setting the .visible property will invoke a draw
        
        # test
        if len(stringList)==1 and isinstance(stringList[0],(tuple,list)):
            stringList = stringList[0]
        for value in stringList:
            if not isinstance(value, basestring):
                raise ValueError("Legend string list should only contain strings.")
        
        # store
        self._stringList = stringList
        
        # clean up labels and lines
        for line in [line for line in self._wobjects]:
            line.Destroy()
        for label in self.children:
            label.Destroy()
        
        # find axes and figure
        axes = self.parent
        while axes and not isinstance(axes, Axes):
            axes = axes.parent
        if not axes:
            return
        fig = axes.GetFigure()
        
        # collect line objects
        lines = []
        twoPoints = False
        for ob in axes._wobjects:
            if len(self._wobjects) >= len(stringList):
                break
            if isinstance(ob, Line):
                # Add line props
                tmp = ob.ls, ob.lc, ob.lw, ob.ms, ob.mc, ob.mw, ob.mec, ob.mew
                lines.append(tmp)
                # Set whether to use two points
                twoPoints = twoPoints or bool(ob.ls and ob.lc and ob.lw)
        
        # create new lines and labels
        maxWidth = 0
        nr = -1
        for lineProps in lines:
            nr += 1
            if nr >= len(stringList):
                break
            # get new line and label
            text = stringList[nr]
            yspacing = self._yspacing * fig._relativeFontSize
            line, label = self._AddLineAndLabel(text, yspacing, twoPoints)
            # apply line properties
            line.ls, line.lc, line.lw = lineProps[0:3]
            line.ms, line.mc, line.mw = lineProps[3:6]
            line.mec, line.mew = lineProps[6:8]
            # correct label size and store max
            deltax, deltay = label.GetVertexLimits()
            label.position.w = (deltax[1]-deltax[0])+2
            maxWidth = max([maxWidth, label.position.w ])
        
        # make own size ok
        if self._wobjects:
            pos = label.position
            self.position.w = maxWidth + pos.x + self._xoffset
            #self.position.h = pos.bottom + self._yoffset
            deltax, deltay = label.GetVertexLimits()
            labelHeight = deltay[1]# - deltay[0]
            self.position.h = pos.top + labelHeight + self._yoffset + 2
            self.visible = True
        else:
            self.visible = False
    
    
    def OnDraw(self):
        
        # draw box
        DraggableBox.OnDraw(self)
        
        # draw lines
        for line in self._wobjects:
            line.OnDraw()
        
        # reset some stuff that was set because it was thinking it was drawing
        # in world coordinates.
        gl.glDisable(gl.GL_DEPTH_TEST)
    
    
    def OnDestroy(self):
        DraggableBox.OnDestroy(self)
        
        # clear lines and such
        for ob in [ob for ob in self._wobjects]:
            ob.Destroy()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import os
import visvis as vv


def ssdfRead(fname, check=False):
    """ Simple function that reads a mesh in the ssdf file format.
    """
    
    # Read structure
    s = vv.ssdf.load(fname)
    
    # Check
    if 'vertices' not in s:
        raise RuntimeError('This ssdf file does not contain mesh data.')
    if 'normals' not in s:
        s.normals = None
    if 'values' not in s:
        s.values = None
    if 'faces' not in s:
        s.faces = None
    if 'verticesPerFace' not in s:
        s.verticesPerFace = 3
    
    # Return mesh object
    return vv.BaseMesh(s.vertices, s.faces, s.normals, s.values, s.verticesPerFace)


def meshRead(fname, check=False):
    """ meshRead(fname, check=False)
    
    Parameters
    ----------
    fname : string
        The name of the file to read.
    check : bool
        For the STL format: if check is True and the file is in ascii,
        some checks to the integrity of the file are done (which is a
        bit slower).
    
    Notes on formats
    ----------------
      * The STL format (.stl) is rather limited in the definition of the
        faces; smooth shading is not possible on an STL mesh.
      * The Wavefront format (.obj) is widely available.
      * For the wavefront format, material, nurbs and other fancy stuff
        is ignored.
      * The SSDF format (.ssdf or .bsdf) is the most efficient in terms
        of memory and speed, but is not widely available.
    
    """
    
    if not os.path.isfile(fname):
        # try loading from the resource dir
        path = vv.core.misc.getResourceDir()
        fname2 = os.path.join(path, fname)
        if os.path.isfile(fname2):
            fname = fname2
        else:
            raise IOError("Mesh file '%s' does not exist." % fname)
    
    # Use file extension to read file
    if fname.lower().endswith('.stl'):
        import visvis.vvio
        readFunc = visvis.vvio.stl.StlReader.read
    elif fname.lower().endswith('.obj'):
        import visvis.vvio
        readFunc = visvis.vvio.wavefront.WavefrontReader.read
    elif fname.lower().endswith('.ssdf') or fname.lower().endswith('.bsdf'):
        readFunc = ssdfRead
    else:
        raise ValueError('meshRead cannot determine file type.')
    
    # Read
    return readFunc(fname, check)

    
if __name__ == '__main__':
    # Create BaseMesh object (has no visualization props)
    bm = vv.meshRead('bunny.ssdf')
    # Show it, returning a Mesh object (which does have visualization props)
    m = vv.mesh(bm)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

def help():
    """ help()
    
    Open a webbrowser with the API documentation of visvis.
    
    Note that all visvis classes and functions have docstrings, so
    typing for example "vv.Mesh?" in IPython or IEP gives you
    documentation on the fly.
    
    """
    import webbrowser
    webbrowser.open("https://github.com/almarklein/visvis/wiki")

if __name__ == '__main__':
    import visvis as vv
    vv.help()



# -*- coding: utf-8 -*-
# flake8: noqa
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

"""
Visvis is a pure Python library for visualization of 1D to 4D data in an
object oriented way. Essentially, visvis is an object oriented layer of
Python on top of OpenGl, thereby combining the power of OpenGl with the
usability of Python. A Matlab-like interface in the form of a set of
functions allows easy creation of objects (e.g. plot(), imshow(), volshow(),
surf()).

With visvis a range of different data can be visualized by simply adding
world objects to a scene (or axes). These world objects can be anything
from plots (lines with markers), to images, 3D rendered volumes,
shaded meshes, or you can program your own world object class. If required,
these data can also be moved in time.

Visvis can be used in Python scripts, interactive Python sessions (as
with IPython or IEP) and can be embedded in applications.

Requirements:
  * Numpy
  * PyOpengl
  * A backend GUI toolkit (PySide, PyQt4, PyQt5, wxPython, GTK, fltk)
  * (optionally, to enable reading and writing of images) imageio

usage:
import visvis as vv

All wobjects, wibjects and functions are present in the visvis
namespace. For clean lists, see vv.wibjects, vv.wobjects, or vv.functions,
respectively.

For more help, see ...
  * the docstrings
  * the examples in the examples dir
  * the examples at the bottom of the function modules (in the functions dir)
  * the online docs: https://github.com/almarklein/visvis/wiki

Visvis is maintained by Almar Klein.

"""

__version__  = '1.13.0'

# Loose sub-modules and sub-packages
from visvis.utils.pypoints import Point, Pointset, Aarray, Quaternion
from visvis.utils import ssdf

# The core
from visvis.core import *

# The wibjects and wobjects
from visvis.wibjects import *
from visvis.wobjects import *

# Expose some more classes
from visvis.wobjects.polygonalModeling import BaseMesh
from visvis.processing.statistics import StatData

# Do this last
from visvis import backends
from visvis.functions import *



# -*- coding: utf-8 -*-
# Copyright (c) 2011, Almar Klein
#
# SSDF is distributed under the terms of the (new) BSD License.
# See http://www.opensource.org/licenses/bsd-license.php

""" Script test_random

Create random ssdf structs, serialize these, read back, and test if
they represent the same data.

"""
import os
import time
import sys
import random
import string
import numpy as np

import ssdf


# From six.py
PY3 = sys.version_info[0] == 3
if PY3:
    string_types = str,
    text_type = str
    binary_type = bytes
    ascii_type = str # Simple string
    unichr = chr
    xrange = range
else:
    string_types = basestring,
    text_type = unicode
    binary_type = str
    ascii_type = str # Simple string


# Get homedir
homedir = '/home/almar/'
if not os.path.isdir(homedir):
    homedir = 'c:/almar/'

# Get printabel chars, add some Unicode characters
CHARS = string.printable + (unichr(169) + unichr(181) + unichr(202) +
                            unichr(1220) + unichr(1138) + unichr(1297))
NAMECHARS = str('abcdefghijklmnopqrstuvwxyz_0123456789')

class Generator:
    def random_object(self, level):
        if level>0:
            id = random.randrange(7) + 1
        else:
            id = random.randrange(5) + 1
        
        if id == 1:
            return None
        elif id == 2:
            return self.random_int()
        elif id == 3:
            return self.random_float()
        elif id == 4:
            return self.random_string()
        elif id == 5:
            return self.random_array()
        elif level>0 and id == 6:
            return self.random_list(level)
        elif level>0 and id == 7:
            return self.random_struct(level)
    
    def random_int(self):
        return random.randint(-2**62, 2**62)
    
    def random_float(self):
        return (random.random()-0.5 ) * 10000
    
    def random_string(self, maxn=16):
        n = random.randrange(0, maxn)
        return ''.join(random.sample(CHARS,n))
    
    def random_array(self, maxn=16):
        # Get array properties
        ndim = random.randrange(0, 4)
        shape = random.sample(xrange(maxn), ndim)
        dtype = random.choice([key for key in ssdf.ssdf_text._DTYPES.keys()])
        # Create array
        if 'int' in dtype:
            return np.random.random_integers(0,100, shape).astype(dtype)
        else:
            return np.random.random_sample(shape).astype(dtype)
    
    def random_struct(self, level, maxn=16):
        level -= 1
        # Create struct and amount of elements
        s = ssdf.new()
        n = random.randrange(0, maxn)
        # Fill
        for i in range(n):
            # Get name
            n = random.randrange(0, maxn)
            name = random.choice(NAMECHARS[:-10])
            name += ''.join(random.sample(NAMECHARS,n))
            if name.startswith('__'):
                name = 'x' + name
            # Get value
            s[name] = self.random_object(level)
        if random.random() > 0.5:
            return s
        else:
            return s.__dict__
    
    def random_list(self, level, maxn=16):
        level -= 1
        # Create list and amount of elements
        items = []
        n = random.randrange(0, maxn)
        # Fill
        for i in range(n):
            # Get value
            items.append( self.random_object(level) )
        if random.random() > 0.5:
            return items
        else:
            return tuple(items)
    
    
    @classmethod
    def create_struct(cls, level=8):
        gen = cls()
        return gen.random_struct(level)


def compare(ob1, ob2, verbose=True):
    """ compare(ob1, ob2, verbose=True)
    
    Compare two (ssdf-compatible) objects. If verbose is True (the default)
    and the objects are not equal, a message is printed indicating the
    first found inequality.
    
    """
    not_equal = ssdf.ssdf_base._not_equal(ob1, ob2)
    if verbose and not_equal:
        print(not_equal)
    return not not_equal


class Tester:
    def __init__(self):
        self._tests = []
        self._stop = False
        self._s = None
    
    def enable_np(self):
        ssdf.np = np
    def disable_np(self):
        ssdf.np = None
    
    def run(self, amount=100000):
        self._stop = False
        t0 = time.time()
        maxtests = amount
        
        while len(self._tests) < maxtests and not self._stop:
            
            # Create struct
            s = Generator.create_struct()
            n = ssdf.count(s)
            self._s = s
            
            # times
            times = []
            sizes = []
            
            # Test text
            t1 = time.time()
            text = ssdf.saves(s)
            times.append(time.time()-t1)
            #
            t1 = time.time()
            s2 = ssdf.loads(text)
            times.append(time.time()-t1)
            #
            if not compare(s, s2):
                print('Test text failed after %i iterations.' % len(self._tests))
                break
            
            # Test binary
            t1 = time.time()
            bb = ssdf.saveb(s)
            times.append(time.time()-t1)
            #
            t1 = time.time()
            s2 = ssdf.loadb(bb)
            times.append(time.time()-t1)
            #
            if not compare(s, s2):
                print('Test bin failed after %i iterations.' % len(self._tests))
                break
            
            # Test text file
            fname = homedir+'projects/ssdftest.ssdf'
            t1 = time.time()
            ssdf.save(fname, s)
            times.append(time.time()-t1)
            #
            t1 = time.time()
            s2 = ssdf.load(fname)
            times.append(time.time()-t1)
            sizes.append(os.stat(fname).st_size)
            #
            if not compare(s, s2):
                print('Test text-file failed after %i iterations.' % len(self._tests))
                break
                
            # Test binary file
            fname = homedir+'projects/ssdftest.bsdf'
            t1 = time.time()
            ssdf.save(fname, s)
            times.append(time.time()-t1)
            #
            t1 = time.time()
            s2 = ssdf.load(fname)
            times.append(time.time()-t1)
            sizes.append(os.stat(fname).st_size)
            #
            if not compare(s, s2):
                print('Test bin-file failed after %i iterations.' % len(self._tests))
                break
            
            # Success
            self._tests.append( (n, times, sizes) )
            print('%i tests successfull' % len(self._tests))
            time.sleep(0.001)
    
    
    def test_numpy(self, amount=10):
    
       
        
        for iter in range(amount):
            
            # Random struct
            s = Generator.create_struct()
            self._s = s
            
            # Text with numpy on writing
            self.enable_np()
            text = ssdf.saves(s)
            #
            self.disable_np()
            s2 = ssdf.loads(text)
            text = ssdf.saves(s2)
            #
            self.enable_np()
            s3 = ssdf.loads(text)
#             if not compare(s, s2):
#                 print('Test text failed after %i iterations.' % iter)
#                 break
            if not compare(s, s3):
                print('Test text failed after %i iterations.' % iter)
                break
            
            # Binary with numpy on writing
            self.enable_np()
            bb = ssdf.saveb(s)
            #
            self.disable_np()
            s2 = ssdf.loadb(bb)
            bb = ssdf.saveb(s2)
            #
            self.enable_np()
            s3 = ssdf.loadb(bb)
#             if not compare(s, s2):
#                 print('Test bin failed after %i iterations.' % iter)
#                 break
            if not compare(s, s3):
                print('Test bin failed after %i iterations.' % iter)
                break
            
            print('%i tests successfull' % iter)
            time.sleep(0.001)
        
#             # Text with numpy on writing
#             self.disable_np()
#             s = Generator.create_struct()
#             text = ssdf.saves(s)
#             #
#             self.enable_np()
#             s2 = ssdf.loads(text)
#             text = ssdf.saves(s2)
#             #
#             self.disable_np()
#             s3 = ssdf.loads(text)
#             if not compare(s, s3):
#                 print('Test text failed after %i iterations.' % iter)
#                 break
#
#             # Binary with numpy on writing
#             self.disable_np()
#             s = Generator.create_struct()
#             bb = ssdf.saveb(s)
#             #
#             self.enable_np()
#             s2 = ssdf.loadb(bb)
#             bb = ssdf.saveb(s2)
#             #
#             self.disable_np()
#             s3 = ssdf.loadb(bb)
#             if not compare(s, s3):
#                 print('Test bin failed after %i iterations.' % iter)
#                 break
        
        # Finish
        self.enable_np()


def find(s, name):
    """ To find an object with a certain name.
    """
    for n in s:
        if ssdf.isstruct(s):
            v = s[n]
        else:
            v = n
            n = None
        
        res = None
        if n == name:
            return v
        elif ssdf.isstruct(v) or isinstance(v, dict):
            res = find(v, name)
        elif isinstance(v, (list, tuple)):
            res = find(v, name)
        elif n:
            print(n)
        if res:
            return res


def show(self):
    
    import visvis as vv
    
    # If there are many tests, make a selection
    if len(self._tests) > 1000:
        tests = random.sample(self._tests, 1000)
    else:
        tests = self._tests
    
    # Get ticks
    nn = [test[0] for test in tests]
    
    # Create figure
    vv.figure(1)
    vv.clf()
    
    # Prepare kwargs
    plotKwargsText = {'ms':'.', 'mc':'b', 'mw':5, 'ls':''}
    plotKwargsBin = {'ms':'.', 'mc':'r', 'mw':5, 'ls':''}
    
    # File size against number of elements
    vv.subplot(221)
    vv.plot(nn, [test[2][0] for test in tests], **plotKwargsText)
    vv.plot(nn, [test[2][1] for test in tests], **plotKwargsBin)
    vv.legend('text', 'binary')
    vv.title('File size')
    
    # Speed against number of elements
    vv.subplot(223)
    vv.plot(nn, [test[1][4] for test in tests], **plotKwargsText)
    vv.plot(nn, [test[1][6] for test in tests], **plotKwargsBin)
    vv.legend('text', 'binary')
    vv.title('Save time')
    
    # Speed (file) against number of elements
    vv.subplot(224)
    vv.plot(nn, [test[1][5] for test in tests], **plotKwargsText)
    vv.plot(nn, [test[1][7] for test in tests], **plotKwargsBin)
    vv.legend('text', 'binary')
    vv.title('Load time')



if __name__ == '__main__':
    # Instantiate tester
    tester = Tester()
    tester.run()



#!/usr/bin/env python
""" This example illustrate different ways to shade mesh objects and
shows an example with lighting. Two light sources are created and
the diffuse reflection of the objects is varied.
"""

import visvis as vv
vv.figure()

# Define shadings to apply, and numbef of steps for diffuse reflection
shading = [ ('plain', None), ('flat', None), ('smooth', None),
            ('smooth', 'plain'), (None, 'smooth')]
ndiffuse = 4


# Init two lights (you should thus see two specular spots)
# Please experiment with the values below to see the effect!

# light0 is always on, and is attached to the camera shining straight ahead
a = vv.gca()
a.light0.ambient = 0.2 # 0.2 is default for light 0
a.light0.diffuse = 1.0 # 1.0 is default

# The other lights are off by default and are positioned at the origin
light1 = a.lights[1]
light1.On()
light1.ambient = 0.0 # 0.0 is default for other lights
light1.color = (1,0,0) # this light is red


# Create spheres
for i in range(len(shading)):
    for j in range(ndiffuse):
        s = vv.solidSphere((i,j,0), (0.3, 0.3, 0.3))
        s.faceShading, s.edgeShading = shading[i]
        s.faceColor = (0.8,0.8,1.0)
        s.edgeColor = (0.8,0.8,1.0)
        s.diffuse = float(j)/ndiffuse

# Set settings for axes
a = vv.gca()
a.axis.xTicks = [str(x) for x in shading]
a.axis.xLabel = 'face- and edgeshading'
a.axis.yTicks = [str(float(j)/ndiffuse) for j in range(ndiffuse)]
a.axis.yLabel = 'diffuse reflection'

# Set back bg
a.bgcolor = 'k'
a.axis.axisColor = 'w'

# Enter mainloop
app = vv.use()
app.Run()



#!/usr/bin/env python
"""
This example illustrates embedding a visvis figure in a wx application.
"""

import wx
import visvis as vv

# Create a visvis app instance, which wraps a wx application object.
# This needs to be done *before* instantiating the main window.
app = vv.use('wx')


class MainWindow(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1, "Embedding in WX", size=(560, 420))
        
        # Make a panel with a button
        self.panel = wx.Panel(self)
        but = wx.Button(self.panel, -1, 'Click me')
        
        # Make figure using "self" as a parent
        Figure = app.GetFigureClass()
        self.fig = Figure(self)
        
        # Make sizer and embed stuff
        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer.Add(self.panel, 1, wx.EXPAND)
        self.sizer.Add(self.fig._widget, 2, wx.EXPAND)
        
        # Make callback
        but.Bind(wx.EVT_BUTTON, self._Plot)
        
        # Apply sizers
        self.SetSizer(self.sizer)
        self.SetAutoLayout(True)
        self.Layout()
        
        # Finish
        self.Show()
    
    
    def _Plot(self, event):
        
        # Make sure our figure is the active one
        # If only one figure, this is not necessary.
        #vv.figure(self.fig.nr)
        
        # Clear it
        vv.clf()
        
        # Plot
        vv.plot([1,2,3,1,6])
        vv.legend(['this is a line'])
        #self.fig.DrawNow()
    

# Two ways to create the application and start the main loop
if True:
    # The visvis way. Will run in interactive mode when used in IEP or IPython.
    app.Create()
    m = MainWindow()
    app.Run()

else:
    # The native way.
    wxApp = wx.App()
    m = MainWindow()
    wxApp.MainLoop()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module shaders_2

Contains the source for various shaders for the Texture2D wobject,
divided in different parts.

The names consists of different parts, seperated by underscores:
  * SH: just to indicate its a shader
  * 2F, 3F, MF, 2V, 3V, MV: the kind of program, 2D/3D texture or mesh,
    vertex or fragment shader. May be left out for some generic parts (such
    as color)
  * part name: can be BASE, STYLE, or anything else really.
  * part name type: Optional. Some parts have alternatives,
    such as the STYLE part for 3D rendering techniques. Or the SHADING
    part for meshes

"""
from visvis.core.shaders import ShaderCodePart


## Color
# These are used by many shaders to be able to deal with scalar or RGB data.
# color1-to-val is used by volume renderes to test thresholds etc.
# color1-to-color2 is used to convert the color sampled from the texture to
# a color to be displayed on screen.

# This one is not really usefull, but included for completeness
SH_COLOR_SCALARNOCMAP = ShaderCodePart('color', 'scalar-nocmap',
"""
    >>--uniforms--
    uniform vec2 scaleBias;
    // --uniforms--
    
    >>--color1-to-val--
    val = color1.r;
    
    >>--color1-to-color2--
    color2.rgb = ( color1.r + scaleBias[1] ) * scaleBias[0];
    color2.a = 1.0;
""")

SH_COLOR_SCALAR = ShaderCodePart('color', 'scalar',
"""
    >>--uniforms--
    uniform sampler1D colormap;
    uniform vec2 scaleBias;
    // --uniforms--
    
    >>--color1-to-val--
    val = color1.r;
    
    >>--color1-to-color2--
    color2 = texture1D( colormap, (color1.r + scaleBias[1]) * scaleBias[0]);
""")

SH_COLOR_RGB = ShaderCodePart('color', 'RGB',
"""
    >>--uniforms--
    uniform vec2 scaleBias;
    // --uniforms--
    
    >>--color1-to-val--
    val = length(color1.rgb);
    
    >>--color1-to-color2--
    color2 = ( color1 + scaleBias[1] ) * scaleBias[0];
""")


## 2D fragment base
# Shows 2D texture and does anti-aliasing when image is zoomed out.

SH_2F_BASE = ShaderCodePart('base', '2D-fragment-default',
"""
    // Uniforms obtained from OpenGL
    uniform sampler2D texture; // The 3D texture
    uniform vec2 shape; // And its shape (as in OpenGl)
    uniform vec2 extent; // Extent of the data in world coordinates
    uniform vec4 aakernel; // The smoothing kernel for anti-aliasing
    // --uniforms--
    
    // Varyings obtained from vertex shader
    // --varyings--
    
    void main()
    {
        // Get centre location
        vec2 pos = gl_TexCoord[0].xy;
        
        // Init value
        vec4 color1 = vec4(0.0, 0.0, 0.0, 0.0);
        vec4 color2; // to set color later
        
        // Init kernel and number of steps
        vec4 kernel = aakernel;
        int sze = 0; // Overwritten in aa-steps part
        
        // Init step size in tex coords
        float dx = 1.0/shape.x;
        float dy = 1.0/shape.y;
        
        // Allow more stuff
        // --pre-loop--
        
        // Convolve
        for (int y=-sze; y<sze+1; y++)
        {
            for (int x=-sze; x<sze+1; x++)
            {
                float k = kernel[int(abs(float(x)))] * kernel[int(abs(float(y)))];
                vec2 dpos = vec2(float(x)*dx, float(y)*dy);
                color1 += texture2D(texture, pos+dpos) * k;
            }
        }
        
        // Allow more stuff
        // --post-loop--
        
        // Determine final color
        // --color1-to-color2--
        gl_FragColor = color2;
        
    }

""")

# This cannot be done with a uniform, because on some systems it wont code
# with loops of which the length is undefined.
_SH_2F_AASTEPS = """
    >>int sze = 0;
    int sze = %i;
"""
SH_2F_AASTEPS_0 = ShaderCodePart('aa-steps', '0', _SH_2F_AASTEPS % 0)
SH_2F_AASTEPS_1 = ShaderCodePart('aa-steps', '1', _SH_2F_AASTEPS % 1)
SH_2F_AASTEPS_2 = ShaderCodePart('aa-steps', '2', _SH_2F_AASTEPS % 2)
SH_2F_AASTEPS_3 = ShaderCodePart('aa-steps', '3', _SH_2F_AASTEPS % 3)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Robert Schroll
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def view(viewparams=None, axes=None, **kw):
    """ view(viewparams=None, axes=None, **kw)
    
    Get or set the view parameters for the given axes.
    
    Parameters
    ----------
    viewparams : dict
        View parameters to set.
    axes : Axes instance
        The axes the view parameters are for.  Uses the current axes by default.
    keyword pairs
        View parameters to set.  These take precidence.
    
    If neither viewparams or any keyword pairs are given, returns the current
    view parameters (as a dict).  Otherwise, sets the view parameters given.
    """
    
    if axes is None:
        axes = vv.gca()
    
    if viewparams or kw:
        axes.SetView(viewparams, **kw)
    else:
        return axes.GetView()


if __name__=='__main__':
    a = vv.gca()
    vv.solidTeapot()
    v = vv.view()
    # ... rotate the figure a bit and then call:
    vv.view(v)
    # Note that a.GetView() and a.SetView(v) are equivalent



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import numpy as np
import time

##
v = np.array([10,11,12,20,21,22,30,31,32,40,41,42,50,51,52,60,61,62])
v.shape = 6,3
N = v.shape[0]
faces = np.arange(N); faces.shape = len(v)//3, 3


def calculateNormals(mesh):
    """ calculateNormals(mesh)
    
    Calculate the normal data from the vertices.
    Handles triangular and quad faces.
    
    """
    t0 = time.time()  # noqa
    
    # Get vertices as np array
    vertices = mesh._vertices
    if vertices is None:
        return
    
    # Init normal array
    N = vertices.shape[0]
    normals = np.zeros((N,3), dtype='float32')
    
    # Get faces array
    faces = mesh._GetFaces()
    _, vpf = faces.shape
    
    # Apply
    
    # Select lists of vertices. v1,v2,v3 are lists of vertices
    # corresponding to the first, second, third vertices of the faces.
    # They are Nfaces times 3/4
    
    if vpf == 3:
        # Get vertex per faces index (for all faces)
        v1 = vertices[faces[:,0]]
        v2 = vertices[faces[:,1]]
        v3 = vertices[faces[:,2]]
        
        # Calculate normals
        _vectorsToNormals(v2-v1, v2-v3, faces, normals)
        
    elif vpf == 4:
        # Get vertex per faces index (for all faces)
        v1 = vertices[faces[:,0]]
        v2 = vertices[faces[:,1]]
        v3 = vertices[faces[:,2]]
        v4 = vertices[faces[:,3]]
        # Calculate normals using all possible sets of 3 vertices.
        # (order found by simply testing)
        _vectorsToNormals(v2-v1, v2-v3, faces, normals)
        _vectorsToNormals(v2-v4, v2-v3, faces, normals)
        _vectorsToNormals(v1-v3, v4-v3, faces, normals)
        _vectorsToNormals(v2-v1, v1-v4, faces, normals)
    
    # Normalize the normals - but avoid NaN
    lengths = normals[:,0]**2 + normals[:,1]**2 + normals[:,2]**2
    lengths = lengths**0.5
    where, = np.where(lengths > 0)
    lengths_where = lengths[where]
    normals[where, 0] /= lengths_where
    normals[where, 1] /= lengths_where
    normals[where, 2] /= lengths_where
    
#     print('%i nans' % np.isnan(normals).sum())
#     print('calculated normals in %1.2 s' % (time.time()-t0))
    
    # Store normals (need to flip sign to follow the right hand rule)
    mesh._normals = -normals


def _vectorsToNormals(a, b, faces, normals):
    
    # The normal is orthogonal to both vectors. Use cross product
    normalsPerFace = np.zeros((faces.shape[0],3), dtype='float32')
    normalsPerFace[:,0] = a[:,1]*b[:,2] - a[:,2]*b[:,1]
    normalsPerFace[:,1] = a[:,2]*b[:,0] - a[:,0]*b[:,2]
    normalsPerFace[:,2] = a[:,0]*b[:,1] - a[:,1]*b[:,0]
    
    # Get number of faces.
    Nfaces = faces.shape[0]
    
    # The normals can be distributed over the normals per vertex
    if Nfaces > 32000:
        # Fast and dirty: if an index in faces occurs twice in the same
        # position (0,1,or 2) then only the final one is added.
        # Thanks to Robert Schroll for pointing this out
        for f in range(faces.shape[1]):
            normals[faces[:,f]] += normalsPerFace
    else:
        # The right (but slower) way. We could make a cython function of this.
        # On the teapot (6000+ faces) this takes less then 0.1 secs on my
        # i3 laptop. Since this only has to be done once for each Mesh object,
        # that's not too bad, but waiting for over a second can become
        # iritating.
        for f in range(faces.shape[1]):
            for i in range(faces.shape[0]):
                normals[faces[i,f]] += normalsPerFace[i]


def calculateNormals_old(mesh):
    """ calculateNormals(mesh)
    
    Calculate the normal data from the vertices.
    Handles triangular and quad faces.
    
    """
    t0 = time.time()
    
    # Get vertices as np array
    vertices = mesh._vertices
    if vertices is None:
        return
    
    # Init normal array
    N = vertices.shape[0]
    normals = np.zeros((N,3), dtype='float32')
    defaultNormal = np.array([1,0,0], dtype='float32')
    
    # For all faces, calculate normals, and add to normals
    # If quads, we neglect the 4th vertex, which should be save, as it
    # should be in the same plane.
    for ii in mesh._IterFaces():
        v1 = vertices[ii[0],:]
        v2 = vertices[ii[1],:]
        v3 = vertices[ii[2],:]
        # Calculate normal
        tmp = np.cross(v1-v2,v2-v3)
        if np.isnan(tmp).sum():
            tmp = defaultNormal
        # Insert normals
        normals[ii[0],:] += tmp
        normals[ii[1],:] += tmp
        normals[ii[2],:] += tmp
    
    # Normalize normals
    for i in range(N):
        tmp = normals[i,:]
        tmp = tmp / ( (tmp**2).sum()**0.5 )
        if np.isnan(tmp).sum():
            tmp = defaultNormal
        normals[i,:] = -tmp
    print('calculated normals in %1.2f s' % (time.time()-t0))
    
    # Store normals
    mesh._normals = normals



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def volshow(*args, **kwargs):
    """ volshow(vol, clim=None, cm=CM_GRAY, axesAdjust=True, axes=None)
    
    Display a 3D image (a volume).
    
    This is a convenience function that calls either volshow3() or
    volshow2(). If the current system supports it (OpenGL version >= 2.0),
    displays a 3D  rendering (volshow3). Otherwise shows three slices
    that can be moved interactively (volshow2).
    
    Parameters
    ----------
    vol : numpy array
        The 3D image to visualize. Can be grayscale, RGB, or RGBA.
        If the volume is an anisotropic array (vv.Aaray), the appropriate
        scale and translate transformations are applied.
    clim : 2-element tuple
        The color limits to scale the intensities of the image. If not given,
        the im.min() and im.max() are used (neglecting nan and inf).
    cm : Colormap
        Set the colormap to apply in case the volume is grayscale.
    axesAdjust : bool
        If axesAdjust==True, this function will call axes.SetLimits(), and set
        the camera type to 3D. If daspectAuto has not been set yet, it is
        set to False.
    axes : Axes instance
        Display the image in this axes, or the current axes if not given.
    
    Any other keyword arguments are passed to either volshow2() or volshow3().
    
    """
    
    # Make sure that a figure exists
    vv.gcf()
    
    # Test and run
    if vv.settings.volshowPreference==3 and vv.misc.getOpenGlCapable(2.0):
        return vv.volshow3(*args, **kwargs)
    else:
        return vv.volshow2(*args, **kwargs)


if __name__ == "__main__":
    vol = vv.aVolume()
    t = vv.volshow(vol)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

from visvis import BaseFigure
import visvis as vv

def gcf():
    """ gcf()
    
    Get the current figure. If there is no figure yet, figure() is
    called to create one. To make a figure current,
    use Figure.MakeCurrent().
    
    See also gca()
    
    """
    
    if not BaseFigure._figures:
        # no figure yet
        return vv.figure()
    
    nr = BaseFigure._currentNr
    if not nr in BaseFigure._figures:
        # erroneous nr
        nr = list(BaseFigure._figures.keys())[0]
        BaseFigure._currentNr = nr
    
    return BaseFigure._figures[nr]

if __name__ == '__main__':
    fig = vv.gcf()
